<?php


do_action( 'builder_start' );


// Load translations
load_theme_textdomain( 'it-l10n-Builder', dirname( __FILE__ ) . '/lang' );

// Load in the classes library
require_once( dirname( __FILE__ ) . '/lib/classes/load.php' );

// Define needed theme global vars
$GLOBALS['theme_menu_var'] = 'ithemes-builder-theme';
$GLOBALS['theme_index'] = 'it-builder';

// Load needed additional functions
require_once( dirname( __FILE__ ) . '/lib/layout-engine/functions.php' );


if ( function_exists( 'add_theme_support' ) ) {
	// Add 2.9+ thumbnail support
	add_theme_support( 'post-thumbnails' );
	
	// Add 3.0+ menu support
	add_theme_support( 'menus' );
	
	// Indicate that this child theme supports specific Builder settings
	// Note that style or template file modifications can cause the settings changes to no longer function
/*	add_theme_support( 'builder-show-hide-categories' );
	add_theme_support( 'builder-show-hide-tags' );
	add_theme_support( 'builder-show-hide-date' );
	add_theme_support( 'builder-customizable-date' );
	add_theme_support( 'builder-show-hide-meta-bottom-comments' );
	add_theme_support( 'builder-show-hide-post-meta-comments' );
	add_theme_support( 'builder-show-hide-author' );
	add_theme_support( 'builder-show-hide-static-front-page-title' );*/
}


// Add RSS feed links
if ( version_compare( $GLOBALS['wp_version'], '2.9.7', '>' ) )
	add_theme_support( 'automatic-feed-links' );
else if ( function_exists( 'automatic_feed_links' ) )
	automatic_feed_links();


// Init code that runs after all libraries have loaded
function it_builder_theme_init() {
	// Load default layouts
	ITUtility::require_file_once( 'default-layouts.php' );
	
	// Setup up the rendering engine
	ITUtility::require_file_once( 'lib/layout-engine/init.php' );
	
	// Theme Options code
	ITUtility::require_file_once( 'lib/theme-options/theme-options.php' );
	
	// Billboard Code
	ITUtility::require_file_once( 'lib/billboard/billboard.php' );
	
	// Feedburner Widget Code
	ITUtility::require_file_once( 'lib/feedburner-widget/feedburner-widget.php' );
	
	// Load Builder Shortcodes
	ITUtility::require_file_once( 'lib/shortcodes/shortcodes.php' );
}
add_action( 'it_libraries_loaded', 'it_builder_theme_init' );


// Certain code must be loaded at specific times
function ithemes_functions_after_init() {
	// Include Tutorials Page
	ITUtility::require_file_once( 'lib/tutorials/tutorials.php' );
}
add_action( 'ithemes_load_plugins', 'ithemes_functions_after_init' );


// Now the text widget supports shortcodes
add_filter( 'widget_text', 'do_shortcode' );


// Apply SEO
function builder_seo_options() {
	global $post, $wp_theme_options;
	
	// build our excerpt
	if ( isset( $post ) ) {
		$post_content = ( strlen( strip_tags( $post->post_content ) ) <= 150) ? strip_tags($post->post_content) : substr(strip_tags($post->post_content),0,150);
		$post_excerpt = ( $post->post_excerpt ) ? $post->post_excerpt : $post_content;
		
		// set the description
		$description = ( is_home() ) ? get_bloginfo( 'description' ) : $post_excerpt;
		$description = str_replace( '"', '&quot;', html_entity_decode( $description ) );
		
		// get the tags
		$post_tags = '';
		if ( isset( $post ) && is_object( $post ) ) {
			$tags = get_the_tags( $post->ID );
			if ( ! empty( $tags ) && is_array( $tags ) ) {
				foreach( (array) $tags as $tag )
					if ( isset( $tag ) && is_object( $tag ) )
						$post_tags .= ','. $tag->name;
			}
			$post_tags = substr( $post_tags, 1 ); // removing the first "," from the list
			
			$post_tags = str_replace( '"', '&quot;', $post_tags );
		}
	}
	
	// add the follow code to our meta section
	echo "\n<!--To follow, or not to follow-->\n";
	if ( is_home() || is_single() || is_page() )
		echo "<meta name='robots' content='index,follow' />\n";
	else if ( ( 'no' !== $wp_theme_options['cat_index'] ) && is_category() )
		echo "<meta name='robots' content='index,follow' />\n";
	else if ( ! is_home() && ! is_single() && ! is_page() )
		echo "<meta name='robots' content='noindex,follow' />\n";
	
	// add the description and keyword code to our meta section
	echo "<!--Add Description and Keywords-->\n";
	if ( ( 'no' !== $wp_theme_options['tag_as_keyword'] ) && is_single() && ! empty( $post_tags ) )
		echo "<meta name='keywords' content=\"$post_tags\" />\n";
	if ( is_home() || is_single() || is_page() )
		echo "<meta name='description' content=\"$description\" />\n";
}
add_action( 'builder_add_meta_data', 'builder_seo_options' );

// Tracking/Analytics Code
function print_tracking() {
	global $wp_theme_options;
	
	if ( isset( $wp_theme_options['tracking'] ) )
		echo stripslashes( $wp_theme_options['tracking'] );
}
function init_print_tracking() {
	global $wp_theme_options;
	
	if ( isset( $wp_theme_options['tracking_pos'] ) && ( 'header' === $wp_theme_options['tracking_pos'] ) )
		add_action( 'wp_head', 'print_tracking' );
	else // default
		add_action( 'builder_layout_engine_render_container', 'print_tracking', 20 );
}
add_action( 'template_redirect', 'init_print_tracking' );


// Temporary Builder title function until Builder SEO can be finished
function builder_add_title() {
	
?>
	<!-- The Title -->
	<title><?php wp_title( '' ); ?><?php if( wp_title( '', false ) ) echo ' :: '; ?><?php bloginfo( 'name' ); ?></title>
<?php
	
}
add_action( 'builder_add_title', 'builder_add_title' );


if ( file_exists( dirname( __FILE__ ) . '/custom-functions.php' ) )
	ITUtility::require_file_once( dirname( __FILE__ ) . '/custom-functions.php' );


function filter_it_tutorials_top_menu_icon( $icon ) {
	it_classes_load( 'it-file-utility.php' );
	
	return ITFileUtility::get_url_from_file( dirname( __FILE__ ) . '/lib/layout-engine/images/builder-icon-16-inactive.png' );
}
add_filter( 'it_tutorials_top_menu_icon', 'filter_it_tutorials_top_menu_icon' );


?>
