<?php

/*
Written by Chris Jean for iThemes.com
Version 2.1.7

Version History
	2.0.0 - 2010-01-07
		Changed all instances of layout_styles to extensions
		Changed all instances of Layout Styles to Extensions
		Changed to _storage_version 1.2
	2.0.1 - 2010-01-07
		Changed "No Layout Style" to "No Extension"
	2.1.0 - 2010-01-14
		Added specific author view options
		Hide set views for views that are no longer available (such as builder_is_blog)
	2.1.1 - 2010-01-17
		Changed builder_layout_engine_get_extension_data to builder_get_extension_data
		Changed builder_layout_engine_get_available_views to builder_get_available_views
		Changed builder_layout_engine_get_extensions to builder_get_extensions
	2.1.2 - 2010-02-05
		Added builder_editor_save_custom_settings and builder_editor_add_custom_settings action to allow
			plugins to add new options to the Layout Editor
	2.1.3 - 2010-02-20
		Changed outdated text layout-styles to extensions
	2.1.4 - 2010-04-26
		Internationalized strings
	2.1.5 - 2010-07-16
		Fixed bug in author view selection
	2.1.6 - 2010-07-22
		Fixed bug when calculating number of views
	2.1.7 - 2010-08-24
		Added global admin styling to supply an image to screen_icon calls
*/


if ( ! class_exists( 'BuilderLayoutEditor' ) ) {
	class BuilderLayoutEditor extends ITCoreClass {
		var $_var = 'layout_settings';
		var $_page_title = '';
		var $_page_var = 'layout-editor';
		var $_menu_title = '';
		var $_storage_version = '1.3';
		
		var $_modules = array();
		
		
		function BuilderLayoutEditor() {
			$this->_page_title = _x( 'Modify Site Layouts', 'page title', 'it-l10n-Builder' );
			$this->_menu_title = _x( 'Layouts', 'menu title', 'it-l10n-Builder' );
			
			
			$this->ITCoreClass();
			
			$this->_file = __FILE__;
			
			ITUtility::require_file_once( 'lib/layout-engine/layout-meta-box.php' );
			
			add_action( 'admin_print_styles', array( $this, 'add_global_admin_styles' ) );
		}
		
		function init() {
			ITCoreClass::init();
			
			$this->_modules = apply_filters( 'builder_get_modules', array() );
		}
		
		function contextual_help( $text, $screen ) {
//			$text = "<h5>Get help with \"Layout Editor\"</h5>\n";
//			$text .= "The Layout Editor allows you to create an unlimited variety of layout options for your site.";
			
			return $text;
		}
		
		function add_admin_scripts() {
			ITCoreClass::add_admin_scripts();
			
			wp_enqueue_script( "{$this->_var}-theme-options", "{$this->_plugin_url}/js/layout-editor.js" );
			wp_enqueue_script( 'scriptaculous-effects' );
			
			do_action( 'builder_module_enqueue_admin_scripts', "{$this->_plugin_url}/modules" );
		}
		
		function add_admin_styles() {
			ITCoreClass::add_admin_styles();
			
			wp_enqueue_style( "{$this->_var}-theme-options", "{$this->_plugin_url}/css/layout-editor.css" );
			
			do_action( 'builder_module_enqueue_admin_styles', "{$this->_plugin_url}/modules" );
		}
		
		function add_global_admin_styles() {
			wp_enqueue_style( "{$this->_var}-global-admin", "{$this->_plugin_url}/css/admin-global.css" );
		}
		
		
		// Pages //////////////////////////////////////
		
		function index() {
			ITCoreClass::index();
			
			if ( ! empty( $_REQUEST['cancel'] ) )
				$this->_list_layouts();
			else if ( ! empty( $_REQUEST['reset_data'] ) )
				$this->_reset_data();
			else if ( ! empty( $_REQUEST['hide_widget_areas'] ) )
				$this->_hide_widget_areas();
			else if ( ! empty( $_REQUEST['show_widget_areas'] ) )
				$this->_show_widget_areas();
			else if ( ! empty( $_REQUEST['modify_module_settings'] ) )
				$this->_modify_module_settings();
			else if ( ! empty( $_REQUEST['delete_layout'] ) )
				$this->_deleteLayout();
			else if ( ! empty( $_REQUEST['delete_layout_screen'] ) )
				$this->_deleteLayoutScreen();
			else if ( ! empty( $_REQUEST['set_default_layout'] ) )
				$this->_setDefaultLayout();
			else if ( ! empty( $_REQUEST['set_default_layout_screen'] ) )
				$this->_setDefaultLayoutScreen();
			else if ( ! empty( $_REQUEST['duplicate_layout'] ) )
				$this->_duplicateLayout();
			else if ( ! empty( $_REQUEST['duplicate_layout_screen'] ) )
				$this->_duplicateLayoutScreen();
			else if ( ! empty( $_REQUEST['delete_view'] ) )
				$this->_delete_view();
			else if ( ! empty( $_REQUEST['delete_view_screen'] ) )
				$this->_delete_view_screen();
			else if ( ! empty( $_REQUEST['modify_view'] ) )
				$this->_modify_view();
			else if ( ! empty( $_REQUEST['modify_view_screen'] ) )
				$this->_modify_view_screen();
			else if ( ! empty( $_REQUEST['modify_views'] ) )
				$this->_modify_views();
			else if ( ! empty( $_REQUEST['add_module'] ) )
				$this->_add_module();
			else if ( ! empty( $_REQUEST['add_module_screen'] ) )
				$this->_add_module_screen();
			else if ( ! empty( $_REQUEST['save'] ) || ! empty( $_REQUEST['save_and_continue'] ) )
				$this->_save_layout();
			else if ( ! empty( $_REQUEST['layout'] ) || ! empty( $_REQUEST['add_layout'] ) )
				$this->_modify_layout();
			else
				$this->_list_layouts();
		}
		
		function _hide_widget_areas() {
			if ( isset( $this->_options['layouts'][$_REQUEST['hide_widget_areas']] ) ) {
				$this->_options['layouts'][$_REQUEST['hide_widget_areas']]['hide_widgets'] = 'yes';
				ITUtility::show_status_message( sprintf( __( 'Widget areas for %s hidden from the <a href="widgets.php">Widgets editor</a>', 'it-l10n-Builder' ), $this->_options['layouts'][$_REQUEST['hide_widget_areas']]['description'] ) );
				
				$this->_save();
			}
			else
				ITUtility::show_error_message( __( 'Unable to find requested layout', 'it-l10n-Builder' ) );
			
			$this->_list_layouts();
		}
		
		function _show_widget_areas() {
			if ( isset( $this->_options['layouts'][$_REQUEST['show_widget_areas']] ) ) {
				$this->_options['layouts'][$_REQUEST['show_widget_areas']]['hide_widgets'] = 'no';
				ITUtility::show_status_message( sprintf( __( 'Widget areas for %s now appear in the <a href="widgets.php">Widgets editor</a>', 'it-l10n-Builder' ), $this->_options['layouts'][$_REQUEST['show_widget_areas']]['description'] ) );
				
				$this->_save();
			}
			else
				ITUtility::show_error_message( __( 'Unable to find requested layout', 'it-l10n-Builder' ) );
			
			$this->_list_layouts();
		}
		
		function _reset_data() {
			global $wpdb;
			
			foreach ( (array) $this->_options['layouts'] as $id => $layout )
				$wpdb->query( $wpdb->prepare( "DELETE FROM {$wpdb->postmeta} WHERE meta_key='_custom_layout' AND meta_value=%s", $id ) );
			
			
			do_action( "it_storage_reset_{$this->_var}" );
			
			ITUtility::show_status_message( __( 'Data reset', 'it-l10n-Builder' ) );
			
			$this->_list_layouts();
		}
		
		function _modify_module_settings( $data = false ) {
			if ( false === $data )
				$data = $_REQUEST;
			
			$module = $data['modify_module_settings'];
			$id = $data['id'];
			$result = array();
			
			if ( isset( $_POST['save'] ) ) {
				if ( method_exists( $this->_modules[$module], 'validate' ) )
					$result = $this->_modules[$module]->validate();
				
				$save_data = ( isset( $result['data'] ) && is_array( $result['data'] ) ) ? $result['data'] : $_POST;
				
				if ( ! isset( $result['errors'] ) ) {
					$this->_save_module_settings( $save_data );
					return;
				}
			}
			
			if ( method_exists( $this->_modules[$module], 'edit' ) ) {
				echo "<div style=\"font-size:16px;font-weight:bold;text-align:center;\">";
				_e( 'Modify Module Settings', 'it-l10n-Builder' );
				echo "</div><br />\n";
				
				$form =& new ITForm( $_POST );
				
				$form->start_form();
				
				$this->_modules[$module]->edit( $form, $result );
				echo "<br />\n";
				
				echo "<div style=\"text-align:center;\">\n";
				
				$form->add_submit( 'save', array( 'value' => 'Save', 'class' => 'button-primary save' ) );
				
				if ( ( '%id' === $data['id'] ) || ( ! empty( $_REQUEST['new_module'] ) ) ) {
					$form->add_submit( 'cancel', array( 'value' => 'Cancel', 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.remove_new_module(); win.tb_remove(); return false;' ) );
					$form->add_hidden( 'new_module', '1' );
				}
				else
					$form->add_submit( 'cancel', array( 'value' => 'Cancel', 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.tb_remove(); return false;' ) );
				
				echo "</div>\n";
				
				$form->add_hidden( 'modify_module_settings', $module );
				$form->add_hidden( 'render_clean', '1' );
				$form->add_hidden( 'id', $id );
				$form->add_hidden( 'layout_width', '0' );
				
				if ( isset( $data['layout'] ) )
					$form->add_hidden( 'layout', $data['layout'] );
				
				$form->end_form();
				
?>
	<?php if ( empty( $result['errors'] ) ) : ?>
		<script type="text/javascript">
			/* <![CDATA[ */
			resize_thickbox_width( "<?php echo $this->_modules[$module]->get_editor_width(); ?>" );
			
			load_module_data("<?php echo $id; ?>");
			/* ]]> */
		</script>
	<?php endif; ?>
<?php
				
				return;
			}
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		close_thickbox();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _save_module_settings( $data ) {
			$module = $_REQUEST['modify_module_settings'];
			$id = $_REQUEST['id'];
			$layout_option = $this->_modules[$module]->get_layout_option();
			
			$form =& new ITForm();
			
			$form->start_form();
			foreach ( (array) $this->_modules[$module]->get_defaults() as $var => $val ) {
				if ( ! isset( $data[$var] ) )
					$data[$var] = '';
				$form->add_hidden( "module-$id-$var", $data[$var] );
			}
			$form->end_form();
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		var win = window.dialogArguments || opener || parent || top;
		
		save_module_data("<?php echo $id; ?>");
		win.update_preview_image("<?php echo "module-$id-$layout_option"; ?>");
		/* ]]> */
	</script>
<?php
			
		}
		
		function _deleteLayout() {
			$this->_add_layout_views();
			
			$layout = $_REQUEST['delete_layout_screen'];
			$description = $this->_options['layouts'][$layout]['description'];
			
			if ( ( $this->_options['layouts'][$layout]['total_num_views'] > 0 ) || ( $layout === $this->_options['default'] ) ) {
				if ( empty( $_POST['replacement_layout'] ) ) {
					ITUtility::show_error_message( __( 'You must select a replacement layout.', 'it-l10n-Builder' ) );
					$this->_deleteLayoutScreen();
					
					return;
				}
				if ( ! is_array( $this->_options['layouts'][$_POST['replacement_layout']] ) || ( $_POST['replacement_layout'] === $layout ) ) {
					ITUtility::show_error_message( __( 'The replacement layout chosen is unavailable. Please select a different replacement layout.', 'it-l10n-Builder' ) );
					$this->_deleteLayoutScreen();
					
					return;
				}
				
				global $wpdb;
				
				$replacement = $_POST['replacement_layout'];
				
				$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->postmeta} SET meta_value=%s WHERE meta_key='_custom_layout' AND meta_value=%s", $replacement, $layout ) );
				
				foreach ( (array) $this->_options['views'] as $id => $view ) {
					if ( $layout === $view['layout'] )
						$this->_options['views'][$id]['layout'] = $replacement;
				}
				
				if ( $this->_options['default'] === $layout )
					$this->_options['default'] = $replacement;
			}
			
			unset( $this->_options['layouts'][$layout] );
			$this->_save();
			
			$this->_add_layout_views();
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		var win = window.dialogArguments || opener || parent || top;
		
		win.jQuery("#entry-<?php echo $layout; ?> .num_page_views").html("0");
		win.jQuery("#entry-<?php echo $layout; ?> .num_post_views").html("0");
		win.jQuery("#entry-<?php echo $layout; ?> .num_views").html("0");
		win.jQuery("#entry-<?php echo $layout; ?> .set_default").html("&nbsp;");
		
		win.jQuery("#entry-<?php echo $layout; ?>").css("background-color", "#FF3333").fadeOut("slow").attr("id", "___<?php echo $layout; ?>");
		
		<?php if ( ! empty( $replacement ) ) : ?>
			win.jQuery("#entry-<?php echo $replacement; ?> .num_page_views").html("<?php echo $this->_options['layouts'][$replacement]['num_page_views']; ?>");
			win.jQuery("#entry-<?php echo $replacement; ?> .num_post_views").html("<?php echo $this->_options['layouts'][$replacement]['num_post_views']; ?>");
			win.jQuery("#entry-<?php echo $replacement; ?> .num_views").html("<?php echo $this->_options['layouts'][$replacement]['num_views']; ?>");
			<?php if ( $this->_options['default'] === $replacement ) : ?>
				win.jQuery("#entry-<?php echo $replacement; ?> .set_default").html("<strong>Yes</strong>");
			<?php endif; ?>
		<?php endif; ?>
		
		win.jQuery("tr[id^='entry-']:even").addClass("alternate");
		win.jQuery("tr[id^='entry-']:odd").removeClass("alternate");
		
		win.tb_remove();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _deleteLayoutScreen() {
			$form =& new ITForm();
			
			if ( count( $this->_options['layouts'] ) < 2 ) {
				ITUtility::show_error_message( __( 'There must be at least one layout. This layout can only be removed after adding additional layouts.', 'it-l10n-Builder' ) );
				echo '<br />';
				echo '<div style="text-align:center;">';
				$form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.tb_remove(); return false;' ) );
				echo '</div>';
				
				return;
			}
			
			
			$this->_add_layout_views();
			
			$layouts = array( '' => '' );
			uksort( $this->_options['layouts'], array( &$this, '_orderedSort' ) );
			foreach ( (array) $this->_options['layouts'] as $id => $layout )
				if ( $id != $_REQUEST['delete_layout_screen'] )
					$layouts[$id] = $layout['description'];
			
			$layout = $_REQUEST['delete_layout_screen'];
			$data = $this->_options['layouts'][$layout];
			
			$form =& new ITForm();
			
?>
	<?php $form->start_form(); ?>
		<?php if ( ( $data['total_num_views'] > 0 ) || ( $this->_options['default'] === $layout ) ) : ?>
			<?php if ( $data['total_num_views'] > 0 ) : ?>
				<div><?php _e( 'This layout is currently being used by:', 'it-l10n-Builder' ); ?></div>
				<?php
					$views = array( 'num_page_views' => _x( 'Page', 'view', 'it-l10n-Builder' ), 'num_post_views' => _x( 'Post', 'view', 'it-l10n-Builder' ), 'num_views' => _x( 'View', 'view', 'it-l10n-Builder' ) );
					
					foreach ( (array) $views as $var => $name )
						if ( $data[$var] > 0 )
							echo "<div style=\"margin-left:10px;\">{$data[$var]} $name" . ( ( $data[$var] > 1 ) ? 's' : '' ) . "</div>";
				?>
				<br />
			<?php endif; ?>
			
			<?php if ( $this->_options['default'] === $layout ) : ?>
				<div><?php _e( 'This layout is the default layout. The selected replacement will be set as the new default.', 'it-l10n-Builder' ); ?></div>
				<br />
			<?php endif; ?>
			
			<div><?php _e( 'Please select a layout to use as a replacement:', 'it-l10n-Builder' ); ?></div>
			<div><?php $form->add_drop_down( 'replacement_layout', array( 'value' => $layouts, 'name' => 'replacement_layout' ) ); ?></div>
		<?php else : ?>
			<div><?php printf( __( 'Please confirm that you would like to delete the <strong>%s</strong> layout.', 'it-l10n-Builder' ), $this->_options['layouts'][$layout]['description'] ); ?></div>
		<?php endif; ?>
		
		<br />
		<div style="text-align:center;">
			<?php $form->add_submit( 'delete_layout', array( 'value' => __( 'Delete', 'it-l10n-Builder' ), 'class' => 'button-primary' ) ); ?>
			<?php $form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.tb_remove(); return false;' ) ); ?>
		</div>
		
		<?php $form->add_hidden( 'delete_layout_screen', $layout ); ?>
		<?php $form->add_hidden( 'render_clean', '1' ); ?>
	<?php $form->end_form(); ?>
<?php
			
		}
		
		function _duplicateLayout() {
			$source = $_POST['duplicate_layout_screen'];
			$description = $_POST['duplicate_name'];
			$id = uniqid( '' );
			
			if ( empty( $description ) ) {
				ITUtility::show_error_message( __( 'You must supply a name for the duplicated layout.', 'it-l10n-Builder' ) );
				$this->_duplicateLayoutScreen();
				return;
			}
			foreach ( (array) $this->_options['layouts'] as $layout ) {
				if ( $description === $layout['description'] ) {
					ITUtility::show_error_message( __( 'A layout with that name already exists. Please enter a different name.', 'it-l10n-Builder' ) );
					$this->_duplicateLayoutScreen();
					return;
				}
			}
			
			$this->_options['layouts'][$id] = $this->_options['layouts'][$source];
			$this->_options['layouts'][$id]['description'] = $description;
			
			$this->_options['layouts'][$id]['guid'] = $id;
			
			foreach ( (array) $this->_options['layouts'][$id]['modules'] as $module_id => $module_data )
				$this->_options['layouts'][$id]['modules'][$module_id]['guid'] = uniqid( '' );
			
			
			$this->_save();
			
			
			$layout = $this->_options['layouts'][$id];
			
			
			if ( isset( $layout['hide_widgets'] ) && ( 'yes' === $layout['hide_widgets'] ) ) {
				$hide_widgets_text = '<strong>' . __( 'Yes', 'it-l10n-Builder' ) . '</strong>';
				$hide_widgets_link_title = __( "Show this layout's widget areas in the Widgets editor", 'it-l10n-Builder' );
				$hide_widgets_link_text = __( 'Show widget areas', 'it-l10n-Builder' );
			}
			else {
				$hide_widgets_text = 'No';
				$hide_widgets_link_title = __( "Hide this layout's widget areas from the Widgets editor", 'it-l10n-Builder' );
				$hide_widgets_link_text = __( 'Hide widget areas', 'it-l10n-Builder' );
			}
			
			
			$extension_title = __( 'This layout does not use an Extension', 'it-l10n-Builder' );
			$extension_text = '&nbsp;';
			
			if ( ! empty( $layout['extension'] ) ) {
				$extension = apply_filters( 'builder_get_extension_data', array( 'directory' => $layout['extension'] ) );
				
				if ( isset( $extension['name'] ) ) {
					$extension_text = $extension['name'];
					
					if ( ! empty( $layout['disable_style'] ) && ( 'yes' === $layout['disable_style'] ) ) {
						$extension_title = sprintf( __( 'Uses only %s for styling', 'it-l10n-Builder' ), $extension_text );
						$extension_text = '<strong>' . __( $extension_text, 'it-l10n-Builder' ) . '</strong>';
					}
					else
						$extension_title = sprintf( __( 'Uses %s to modify the default theme styling', 'it-l10n-Builder' ), $extension_text );
				}
			}
			
			$default_title_description = __( 'The default layout is used for all views that don\'t have a specific view set. New layouts inherit the default layout\'s background options.', 'it-l10n-Builder' );
			$hide_widgets_description = __( 'Widget areas for layouts can be hidden from the Widgets editor to make management of widgets easier.', 'it-l10n-Builder' );
			$modify_layout_settings_title = __( 'Modify Layout Settings', 'it-l10n-Builder' );
			$edit_link_text = __( 'Edit', 'it-l10n-Builder' );
			$duplicate_layout_title = __( 'Duplicate Layout', 'it-l10n-Builder' );
			$duplicate_layout_text = __( 'Duplicate', 'it-l10n-Builder' );
			$delete_layout_title = __( 'Delete Layout', 'it-l10n-Builder' );
			$delete_layout_text = __( 'Delete', 'it-l10n-Builder' );
			$page_views_title = sprintf( __( 'The number of pages using %s as the Custom Layout', 'it-l10n-Builder' ), $description );
			$post_views_title = sprintf( __( 'The number of posts using %s as the Custom Layout', 'it-l10n-Builder' ), $description );
			$views_title = sprintf( __( 'The number of views using %s', 'it-l10n-Builder' ), $description );
			
			
			$description = str_replace( '\'', '\\\'', $description );
			$hide_widgets_text = str_replace( "'", "\\'", $hide_widgets_text );
			$hide_widgets_link_title = str_replace( "'", "\\'", $hide_widgets_link_title );
			$hide_widgets_link_text = str_replace( "'", "\\'", $hide_widgets_link_text );
			$extension_title = str_replace( "'", "\\'", $extension_title );
			$default_title_description = str_replace( "'", "\\'", $default_title_description );
			$hide_widgets_description = str_replace( "'", "\\'", $hide_widgets_description );
			$modify_layout_settings_title = str_replace( "'", "\\'", $modify_layout_settings_title );
			$edit_link_text = str_replace( "'", "\\'", $edit_link_text );
			$duplicate_layout_title = str_replace( "'", "\\'", $duplicate_layout_title );
			$duplicate_layout_text = str_replace( "'", "\\'", $duplicate_layout_text );
			$delete_layout_title = str_replace( "'", "\\'", $delete_layout_title );
			$delete_layout_text = str_replace( "'", "\\'", $delete_layout_text );
			
			$page_views_title = str_replace( "'", "\\'", $page_views_title );
			$post_views_title = str_replace( "'", "\\'", $post_views_title );
			$views_title = str_replace( "'", "\\'", $views_title );
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		var win = window.dialogArguments || opener || parent || top;
		
		var newRow = '<tr style="display:none;" id="entry-<?php echo $id; ?>">';
		newRow += '<td>';
		newRow += '<strong><a href="<?php echo $this->_self_link; ?>&layout=<?php echo $id; ?>" title="<?php echo $modify_layout_settings_title; ?>"><?php echo $description; ?></a></strong><br />';
		newRow += '<div class="row-actions">';
		newRow += '<span class="edit"><a href="<?php echo $this->_self_link; ?>&layout=<?php echo $id; ?>" title="<?php echo $modify_layout_settings_title; ?>"><?php echo $edit_link_text; ?></a> | </span>';
		newRow += '<span class="duplicate"><a href="<?php echo $this->_self_link; ?>&render_clean=1&duplicate_layout_screen=<?php echo $id; ?>&TB_iframe=true&height=100&width=250" title="<?php echo $duplicate_layout_title; ?>"><?php echo $duplicate_layout_text; ?></a> | </span>';
		newRow += '<span class="delete"><a href="<?php echo $this->_self_link; ?>&render_clean=1&delete_layout_screen=<?php echo $id; ?>&TB_iframe=true&height=100&width=300" title="<?php echo $delete_layout_title; ?>"><?php echo $delete_layout_text; ?></a></span>';
		newRow += '</div>';
		newRow += '</td>';
		newRow += '<td class="set_default" title="<?php echo $default_title_description; ?>">';
		newRow += '<div class="row-actions">';
		newRow += '<a href="<?php echo $this->_self_link; ?>&render_clean=1&set_default_layout_screen=<?php echo $id; ?>&TB_iframe=true&height=100&width=250">Set as default</a>';
		newRow += '</div>';
		newRow += '</td>';
		newRow += '<td class="widget_areas_hidden" title="<?php echo $hide_widgets_description; ?>">';
		newRow += '<?php echo $hide_widgets_text; ?>';
		newRow += '<div class="row-actions">';
		newRow += '<a href="/wp-admin/admin.php?page=layout-editor&hide_widget_areas=<?php echo $id; ?>" title="<?php echo $hide_widgets_link_title; ?>"><?php echo $hide_widgets_link_text; ?></a>';
		newRow += '</div>';
		newRow += '</td>';
		newRow += '<td class="extension" title="<?php echo $extension_title; ?>"><?php echo $extension_text; ?></td>';
		newRow += '<td class="num_page_views" title="<?php echo $page_views_title; ?>">';
		newRow += '0';
		newRow += '</td>';
		newRow += '<td class="num_post_views" title="<?php echo $post_views_title; ?>">';
		newRow += '0';
		newRow += '</td>';
		newRow += '<td class="num_views" title="<?php echo $views_title; ?>">';
		newRow += '0';
		newRow += '</td>';
		newRow += '</tr>';
		
		var rows = win.jQuery("tr[id^='entry-']");
		var i;
		for(i = 0; i < rows.get().length; i++) {
			if("<?php echo strtolower( $description ); ?>" < win.jQuery("tr[id^='entry-']:eq(" + i + ") a[title='<?php echo $modify_layout_settings_title; ?>']").html().toLowerCase()) {
				break;
			}
		}
		
		i--;
		
		if((rows.get().length > 0) && (i >= 0)) {
			win.jQuery("tr[id^='entry-']:eq(" + i + ")").after(newRow);
		}
		else {
			if(win.jQuery("table#layouts > tbody") == undefined) {
				win.jQuery("table#layouts").html(newRow);
			}
			else {
				win.jQuery("table#layouts > tbody").prepend(newRow);
			}
		}
		
		win.jQuery("tr[id^='entry-']:even").addClass("alternate");
		win.jQuery("tr[id^='entry-']:odd").removeClass("alternate");
		
		var origColor = win.jQuery("#entry-<?php echo $id; ?>").css("background-color");
		win.jQuery("#entry-<?php echo $id; ?>").css("background-color", "#33FF33").fadeIn("slow").animate({backgroundColor:origColor}, 300);
		
		win.tb_init("#entry-<?php echo $id; ?> a[href*='TB_iframe']");
		win.tb_remove();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _duplicateLayoutScreen() {
			$layout = $_REQUEST['duplicate_layout_screen'];
			
			$data = array();
			if ( isset( $_REQUEST['duplicate_name'] ) )
				$data['duplicate_name'] = $_REQUEST['duplicate_name'];
			
			$form =& new ITForm( $data );
			
?>
	<?php $form->start_form(); ?>
		<div><?php printf( __( 'Duplicating <strong>%s</strong>.', 'it-l10n-Builder' ), $this->_options['layouts'][$layout]['description'] ); ?></div>
		<br />
		
		<div>Please name the new layout:</div>
		<div><?php $form->add_text_box( 'duplicate_name' ); ?></div>
		<br />
		
		<div style="text-align:center;">
			<?php $form->add_submit( 'duplicate_layout', array( 'value' => __( 'Create Duplicate', 'it-l10n-Builder' ), 'class' => 'button-primary' ) ); ?>
			<?php $form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.tb_remove(); return false;' ) ); ?>
		</div>
		
		<?php $form->add_hidden( 'duplicate_layout_screen', $layout ); ?>
		<?php $form->add_hidden( 'render_clean', '1' ); ?>
	</form>
<?php
			
		}
		
		function _setDefaultLayout() {
			$layout = $_REQUEST['set_default_layout_screen'];
			$description = $this->_options['layouts'][$layout]['description'];
			
			$original_default = $this->_options['default'];
			
			$this->_options['default'] = $layout;
			$this->_save();
			
			$set_default_link = "<div class='row-actions'><a href='{$this->_self_link}&render_clean=1&set_default_layout_screen=$original_default&TB_iframe=true&height=100&width=250'>";
			$set_default_link .= __( 'Set as default', 'it-l10n-Builder' );
			$set_default_link .= "</a></div>";
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		var win = window.dialogArguments || opener || parent || top;
		
		win.jQuery("#entry-<?php echo $original_default; ?> .set_default").html("<?php echo $set_default_link; ?>");
		win.jQuery("#entry-<?php echo $layout; ?> .set_default").html("<strong><?php _e( 'Yes', 'it-l10n-Builder' ); ?></strong>");
		
		win.tb_init("#entry-<?php echo $original_default; ?> a[href*='set_default_layout_screen']");
		win.tb_remove();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _setDefaultLayoutScreen() {
			$layout = $_REQUEST['set_default_layout_screen'];
			
			$form =& new ITForm();
			
?>
	<?php $form->start_form(); ?>
		<div><?php printf( __( 'Please confirm that you would like to set <strong>%s</strong> as the default layout.', 'it-l10n-Builder' ), $this->_options['layouts'][$layout]['description'] ); ?></div>
		<br />
		
		<div style="text-align:center;">
			<?php $form->add_submit( 'set_default_layout', array( 'value' => __( 'Set as Default', 'it-l10n-Builder' ), 'class' => 'button-primary' ) ); ?>
			<?php $form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.tb_remove(); return false;' ) ); ?>
		</div>
		
		<?php $form->add_hidden( 'set_default_layout_screen', $layout ); ?>
		<?php $form->add_hidden( 'render_clean', '1' ); ?>
	<?php $form->end_form(); ?>
<?php
			
		}
		
		function _add_module() {
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		var win = window.dialogArguments || opener || parent || top;
		
		win.add_module("<?php echo $_POST['module']; ?>");
		
/*		win.tb_remove();*/
		/* ]]> */
	</script>
<?php
			
//			return;
			
			$data = array ( 'modify_module_settings' => $_POST['module'], 'id' => '%id' );
			
			$this->_modify_module_settings( $data );
		}
		
		function _add_module_screen() {
			ksort( $this->_modules );
			
			$form =& new ITForm();
			
?>
	<?php $form->start_form(); ?>
		<div style="font-size:14px;font-weight:bold;text-align:center;"><?php _e( 'Select a module to add to the layout', 'it-l10n-Builder' ); ?></div>
		<table style="border-spacing:0px 10px;">
			<?php $selected = " checked"; ?>
			<?php foreach ( (array) $this->_modules as $module_id => $module ) : ?>
				<tr style="vertical-align:top;" class="add-module-<?php echo $module_id; ?>">
					<td style="padding-right:5px;"><input type="radio" name="module" value="<?php echo $module_id; ?>" id="module-<?php echo $module_id; ?>"<?php echo $selected; ?> /></td>
					<td>
						<label for="module-<?php echo $module_id; ?>" style="display:block;">
							<strong><?php echo $module->_name; ?></strong><br />
							<?php echo $module->_description; ?>
						</label>
					</td>
				</tr>
				<?php $selected = ''; ?>
			<?php endforeach; ?>
		</table>
		<br />
		
		<div style="text-align:center;">
			<?php $form->add_submit( 'add_module', array( 'value' => __( 'Add Module', 'it-l10n-Builder' ), 'class' => 'button-primary' ) ); ?>
			<?php $form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.tb_remove(); return false;' ) ); ?>
		</div>
		
		<?php $form->add_hidden( 'render_clean', '1' ); ?>
	<?php $form->end_form(); ?>
	
	<script type="text/javascript">
		/* <![CDATA[ */
		add_module_screen_hide_maxed_modules();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _list_layouts() {
			uksort( $this->_options['layouts'], array( &$this, '_orderedSort' ) );
			
			$this->_add_layout_views();
			
			$bulk_actions = array(
				''						=> 'Bulk Actions',
				'delete_layout_screen'	=> 'Delete',
				'show_widget_areas'		=> 'Show Widget Areas',
				'hide_widget_areas'		=> 'Hide Widget Areas',
			);
			
			
			$form =& new ITForm();
			
?>
	<div class="wrap">
		<?php $form->start_form(); ?>
			<?php screen_icon(); ?>
			<h2><?php _e( 'Layouts', 'it-l10n-Builder' ); ?></h2>
			
			<div class="tablenav">
				<div class="alignleft actions">
					<?php //$form->add_drop_down( 'bulk_action_1', array( 'value' => $bulk_actions, 'class' => 'no-auto-focus' ) ); ?>
					<?php //$form->add_button( 'submit_bulk_action_1', array( 'value' => 'Apply', 'class' => 'button-secondary bulk-action-submit' ) ); ?>
					<?php $form->add_submit( 'add_layout', array( 'value' => __( 'Create Layout', 'it-l10n-Builder' ), 'class' => 'button-secondary add' ) ); ?>
				</div>
				
				<br class="clear" />
			</div>
			
			<br class="clear" />
			
			<table id="layouts" class="widefat fixed" cellspacing="0">
				<thead>
					<tr class="thead">
<!--						<th scope="col" class="check-column"><input type="checkbox" id="check-all-groups" /></th>-->
						<th><?php _e( 'Layout Description', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The default layout is used for all views that don\'t have a specific view set.', 'it-l10n-Builder' ); ?>"><?php _e( 'Default Layout', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'Widget areas for layouts can be hidden from the Widgets editor to make management of widgets easier.', 'it-l10n-Builder' ); ?>"><?php _e( 'Hide Widget Areas', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'Listings in bold override the default style.css.', 'it-l10n-Builder' ); ?>"><?php _e( 'Extension', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The number of pages using the layout as the Custom Layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Pages', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The number of posts using the layout as the Custom Layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Posts', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The number of views using the layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Views', 'it-l10n-Builder' ); ?></th>
					</tr>
				</thead>
				<tfoot>
					<tr class="thead">
<!--						<th scope="col" class="check-column"><input type="checkbox" id="check-all-groups" /></th>-->
						<th><?php _e( 'Layout Description', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The default layout is used for all views that don\'t have a specific view set.', 'it-l10n-Builder' ); ?>"><?php _e( 'Default Layout', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'Widget areas for layouts can be hidden from the Widgets editor to make management of widgets easier.', 'it-l10n-Builder' ); ?>"><?php _e( 'Hide Widget Areas', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'Listings in bold override the default style.css.', 'it-l10n-Builder' ); ?>"><?php _e( 'Extension', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The number of pages using the layout as the Custom Layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Pages', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The number of posts using the layout as the Custom Layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Posts', 'it-l10n-Builder' ); ?></th>
						<th title="<?php _e( 'The number of views using the layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Views', 'it-l10n-Builder' ); ?></th>
					</tr>
				</tfoot>
				<tbody>
					<?php $class = ' class="alternate"'; ?>
					<?php foreach ( (array) $this->_options['layouts'] as $id => $layout ) : ?>
						<tr id="entry-<?php echo $id; ?>"<?php echo $class; ?>>
<!--							<th scope="row" class="check-column"><input type="checkbox" name="layouts[]" class="administrator layouts" value="<?php echo $id; ?>" /></th>-->
							<td>
								<strong><a href="<?php echo $this->_self_link; ?>&layout=<?php echo $id; ?>" title="<?php _e( 'Modify Layout Settings', 'it-l10n-Builder' ); ?>"><?php echo $layout['description']; ?></a></strong><br />
								<div class="row-actions">
									<span class="edit"><a href="<?php echo $this->_self_link; ?>&layout=<?php echo $id; ?>" title="<?php _e( 'Modify Layout Settings', 'it-l10n-Builder' ); ?>"><?php _e( 'Edit', 'it-l10n-Builder' ); ?></a> | </span>
									<span class="duplicate"><a href="<?php echo $this->_self_link; ?>&render_clean=1&duplicate_layout_screen=<?php echo $id; ?>&TB_iframe=true&height=100&width=250" title="<?php _e( 'Duplicate Layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Duplicate', 'it-l10n-Builder' ); ?></a> | </span>
									<span class="delete"><a href="<?php echo $this->_self_link; ?>&render_clean=1&delete_layout_screen=<?php echo $id; ?>&TB_iframe=true&height=100&width=<?php echo ( $layout['total_num_views'] > 0 ) ? '320' : '300'; ?>" title="<?php _e( 'Delete Layout', 'it-l10n-Builder' ); ?>"><?php _e( 'Delete', 'it-l10n-Builder' ); ?></a></span>
								</div>
							</td>
							<td class="set_default" title="<?php _e( 'The default layout is used for all views that don\'t have a specific view set.', 'it-l10n-Builder' ); ?>">
								<?php if ( $this->_options['default'] == $id ) : ?>
									<strong><?php _e( 'Yes', 'it-l10n-Builder' ); ?></strong>
								<?php else : ?>
									<div class="row-actions">
										<a href="<?php echo $this->_self_link; ?>&render_clean=1&set_default_layout_screen=<?php echo $id; ?>&TB_iframe=true&height=100&width=250" title="<?php _e( 'Set this layout as the default', 'it-l10n-Builder' ); ?>"><?php _e( 'Set as default', 'it-l10n-Builder' ); ?></a>
									</div>
								<?php endif; ?>
							</td>
							<td class="widget_areas_hidden" title="<?php _e( 'Widget areas for layouts can be hidden from the Widgets editor to make management of widgets easier.', 'it-l10n-Builder' ); ?>">
								<?php if ( isset( $layout['hide_widgets'] ) && ( 'yes' === $layout['hide_widgets'] ) ) : ?>
									<strong><?php _e( 'Yes', 'it-l10n-Builder' ); ?></strong>
									<div class="row-actions">
										<a href="<?php echo $this->_self_link; ?>&show_widget_areas=<?php echo $id; ?>" title="<?php _e( 'Show this layout\'s widget areas in the Widgets editor', 'it-l10n-Builder' ); ?>"><?php _e( 'Show widget areas', 'it-l10n-Builder' ); ?></a>
									</div>
								<?php else : ?>
									<?php _e( 'No', 'it-l10n-Builder' ); ?>
									<div class="row-actions">
										<a href="<?php echo $this->_self_link; ?>&hide_widget_areas=<?php echo $id; ?>" title="<?php _e( 'Hide this layout\'s widget areas from the Widgets editor', 'it-l10n-Builder' ); ?>"><?php _e( 'Hide widget areas', 'it-l10n-Builder' ); ?></a>
									</div>
								<?php endif; ?>
							</td>
							<?php
								$extension_title = __( 'This layout does not use a Extension', 'it-l10n-Builder' );
								$extension_text = '&nbsp;';
								
								if ( ! empty( $layout['extension'] ) ) {
									$extension = apply_filters( 'builder_get_extension_data', array( 'directory' => $layout['extension'] ) );
									
									if ( isset( $extension['name'] ) ) {
										$extension_text = $extension['name'];
										
										if ( ! empty( $layout['disable_style'] ) && ( 'yes' === $layout['disable_style'] ) ) {
											$extension_title = sprintf( __( 'Uses only %s for styling', 'it-l10n-Builder' ), $extension_text );
											$extension_text = "<strong>$extension_text</strong>";
										}
										else
											$extension_title = sprintf( __( 'Uses %s to modify the default theme styling', 'it-l10n-Builder' ), $extension_text );
									}
								}
							?>
							<td class="extension" title="<?php echo $extension_title; ?>">
								<?php echo $extension_text; ?>
							</td>
							<td class="num_page_views" title="<?php printf( __( 'The number of pages using %s as the Custom Layout', 'it-l10n-Builder' ), $layout['description'] ); ?>">
								<?php echo $layout['num_page_views']; ?>
							</td>
							<td class="num_post_views" title="<?php printf( __( 'The number of posts using %s as the Custom Layout', 'it-l10n-Builder' ), $layout['description'] ); ?>">
								<?php echo $layout['num_post_views']; ?>
							</td>
							<td class="num_views" title="<?php printf( __( 'The number of views using %s', 'it-l10n-Builder' ), $layout['description'] ); ?>">
								<?php echo $layout['num_views']; ?>
							</td>
						</tr>
						<?php $class = ( $class == '' ) ? ' class="alternate"' : ''; ?>
					<?php endforeach; ?>
				</tbody>
			</table>
			
			<br class="clear" />
			
			<div class="tablenav">
				<div class="alignleft actions">
					<?php $form->add_submit( 'add_layout', array( 'value' => __( 'Create Layout', 'it-l10n-Builder' ), 'class' => 'button-secondary add' ) ); ?>
				</div>
				
				<br class="clear" />
			</div>
			
			<h2><?php _e( 'Apply Layouts to Specific Views', 'it-l10n-Builder' ); ?></h2>
			
			<p><?php _e( 'Your custom layouts can be applied to specific posts or pages. While modify/creating the post or page, simply select the desired layout from the "Custom Layout" box.', 'it-l10n-Builder' ); ?></p>
			<p><?php _e( 'Layouts aren\'t limited to just specific posts and pages. Click the button below to apply your custom layouts to specific views on your site (home page, default post layout, etc).', 'it-l10n-Builder' ); ?></p>
			<p><?php $form->add_submit( 'modify_views', array( 'value' => __( 'Configure Views', 'it-l10n-Builder' ), 'class' => 'button-secondary modify-views' ) ); ?></p>
		<?php $form->end_form(); ?>
	</div>
	
	<script type="text/javascript">
		/* <![CDATA[ */
		jQuery(document).ready( function() { tb_init("a[href*='TB_iframe']"); } );
		/* ]]> */
	</script>
<?php
			
		}
		
		function _delete_view() {
			unset( $this->_options['views'][$_POST['view']] );
			$this->_save();
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		var win = window.dialogArguments || opener || parent || top;
		win.remove_view("<?php echo $_POST['view']; ?>");
		/* ]]> */
	</script>
<?php
			
		}
		
		function _delete_view_screen() {
			$available_views = apply_filters( 'builder_get_available_views', array() );
			
			$form = new ITForm();
			
			$view = $this->_get_view_data( $_REQUEST['delete_view_screen'] );
			
?>
	<?php $form->start_form(); ?>
		<div><?php printf( __( 'Please confirm that you would like to remove the view customization for <strong>%s</strong>.', 'it-l10n-Builder' ), $view['name'] ); ?></div>
		<br />
		
		<div style="text-align:center;">
			<?php $form->add_submit( 'delete_view', array( 'value' => __( 'Remove', 'it-l10n-Builder' ), 'class' => 'button-primary' ) ); ?>
			<?php $form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.tb_remove(); return false;' ) ); ?>
		</div>
		<?php $form->add_hidden( 'view', $_REQUEST['delete_view_screen'] ); ?>
		<?php $form->add_hidden( 'render_clean', '1' ); ?>
	<?php $form->end_form(); ?>
<?php
			
		}
		
		function _modify_view() {
			$errors = array();
			
			if ( empty( $_POST['view'] ) )
				$errors[] = "You must select a View";
			if ( empty( $_POST['layout'] ) )
				$errors[] = "You must select a Layout";
			
			if ( ! empty( $errors ) ) {
				$this->_modify_view_screen( $errors );
				return;
			}
			
			
			$available_views = apply_filters( 'builder_get_available_views', array() );
			
			if ( ! is_array( $this->_options['layouts'][$_POST['layout']] ) || ! is_array( $available_views[$_POST['view']] ) ) {
				ITUtility::show_error_message( __( 'Received bad data. Either the requested layout or view is missing.', 'it-l10n-Builder' ) );
				
				$form->add_hidden( 'cancel', __( 'Close', 'it-l10n-Builder' ) );
				
				return;
			}
			
			if ( ! empty( $_POST['original_view'] ) && ( $_POST['original_view'] !== $_POST['view'] ) ) {
				$original_layout = $this->_options['views'][$_POST['original_view']];
				unset( $this->_options['views'][$_POST['original_view']] );
			}
			
			
			$view = $_POST['view'];
			
			if ( ( 'is_category' === $view ) && ! empty( $_POST['category_id'] ) )
				$view = "{$view}__{$_POST['category_id']}";
			else if ( ( 'is_tag' === $view ) && ! empty( $_POST['tag_id'] ) )
				$view = "{$view}__{$_POST['tag_id']}";
			else if ( ( 'is_author' === $view ) && ! empty( $_POST['author_id'] ) )
				$view = "{$view}__{$_POST['author_id']}";
			
			
			$this->_options['views'][$view] = $_POST['layout'];
			
			$this->_save();
			
			
			$view = $this->_get_view_data( $view );
			
			foreach ( (array) $view as $var => $val ) {
				$view[$var] = str_replace( '\\', '\\\\', $view[$var] );
				$view[$var] = str_replace( '\'', '\\\'', $view[$var] );
			}
			
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		var win = window.dialogArguments || opener || parent || top;
		
		<?php if ( ! empty( $_POST['original_view'] ) ) : ?>
			win.remove_view("<?php echo $_POST['original_view']; ?>");
		<?php endif; ?>
		win.add_view(<?php echo "'{$view['view_id']}', '{$view['name']}', '{$view['description']}', '{$view['layout']}', '{$view['layout_description']}'"; ?>);
		
		close_thickbox();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _modify_view_screen( $errors = array() ) {
			echo "<div style=\"font-size:16px;font-weight:bold;text-align:center;\">";
			echo ( empty( $_REQUEST['original_view'] ) ) ? __( 'Add View', 'it-l10n-Builder' ) : __( 'Modify View', 'it-l10n-Builder' );
			echo "</div><br />\n";
			
			foreach ( (array) $errors as $error )
				ITUtility::show_error_message( $error );
			
			$view = array();
			if ( isset( $_POST['layout'] ) ) {
				$view = array(
					'layout'		=> $_POST['layout'],
					'view'			=> $_POST['view'],
					'category_id'	=> $_POST['category_id'],
					'tag_id'		=> $_POST['tag_id'],
					'author_id'		=> $_POST['author_id'],
				);
			}
			else if ( isset( $_REQUEST['original_view'] ) && isset( $this->_options['views'][$_REQUEST['original_view']] ) )
				$view = $this->_get_view_data( $_REQUEST['original_view'] );
			
			
			if ( isset( $view['layout'] ) )
				$layouts = array( '' => '' );
			uksort( $this->_options['layouts'], array( &$this, '_orderedSort' ) );
			foreach ( (array) $this->_options['layouts'] as $id => $layout )
				$layouts[$id] = $layout['description'];
			
			$available_views = apply_filters( 'builder_get_available_views', array() );
			
			if ( empty( $_REQUEST['original_view'] ) && empty( $_REQUEST['view'] ) )
				$views = array( '' => '' );
			foreach ( (array) $available_views as $id => $view_data )
				if ( empty( $this->_options['views'][$id] ) || ( isset( $_REQUEST['original_view'] ) && ( $_REQUEST['original_view'] === $id ) ) || in_array( $id, array( 'is_category', 'is_tag', 'is_author' ) ) )
					$views[$id] = $view_data['name'];
			asort( $views );
			
			
			$exclude_categories = '';
			$exclude_tags = '';
			$exclude_authors = array();
			
			foreach ( (array) $this->_options['views'] as $view_id => $view_data ) {
				if ( preg_match( '/is_category__(\d+)/', $view_id, $matches ) && ( empty( $view['view_id'] ) || ( $view_id !== $view['view_id'] ) ) ) {
					if ( ! empty( $exclude_categories ) )
						$exclude_categories .= ',';
					$exclude_categories .= $matches[1];
				}
				else if ( preg_match( '/is_tag__(\d+)/', $view_id, $matches ) && ( empty( $view['view_id'] ) || ( $view_id !== $view['view_id'] ) ) ) {
					if ( ! empty( $exclude_tags ) )
						$exclude_tags .= ',';
					$exclude_tags .= $matches[1];
				}
				else if ( preg_match( '/is_author__(\d+)/', $view_id, $matches ) && ( empty( $view['view_id'] ) || ( $view_id !== $view['view_id'] ) ) ) {
					$exclude_authors[] = $matches[1];
				}
			}
			
			
			$category_arguments = array( 'hide_empty' => 0, 'orderby' => 'name', 'hierarchical' => true, 'exclude' => $exclude_categories );
			if ( isset( $view['category_id'] ) && ! is_null( $view['category_id'] ) )
				$category_arguments['selected'] = $view['category_id'];
			
			$category_options = array();
			if ( ! isset( $this->_options['views']['is_category'] ) || ( isset( $view['view_id'] ) && ( 'is_category' === $view['view_id'] ) ) )
				$category_options[''] = __( 'All Categories', 'it-l10n-Builder' );
			
			$categories = get_categories( $category_arguments );
			foreach ( (array) $categories as $category )
				$category_options[$category->term_id] = $this->_get_full_term_name( $category->term_id, $category->taxonomy );
			
			asort( $category_options );
			
			if ( empty( $category_options ) )
				unset( $views['is_category'] );
			
			
			$tag_arguments = array( 'hide_empty' => 0, 'orderby' => 'name', 'exclude' => $exclude_tags );
			
			$tag_options = array();
			if ( ! isset( $this->_options['views']['is_tag'] ) || ( isset( $view['view_id'] ) && ( 'is_tag' === $view['view_id'] ) ) )
				$tag_options[''] = __( 'All Tags', 'it-l10n-Builder' );
			
			$tags = get_tags( array( 'exclude' => $exclude_tags ) );
			foreach ( (array) $tags as $tag )
				$tag_options[$tag->term_id] = $tag->name;
			
			asort( $tag_options );
			
			if ( empty( $tag_options ) )
				unset( $views['is_tag'] );
			
			
			$author_options = array();
			if ( ! isset( $this->_options['views']['is_author'] ) || ( isset( $view['view_id'] ) && ( 'is_author' === $view['view_id'] ) ) )
				$author_options[''] = __( 'All Authors', 'it-l10n-Builder' );
			
			foreach ( (array) get_author_user_ids() as $author_id ) {
				if ( ! in_array( $author_id, $exclude_authors ) ) {
					$author = get_userdata( $author_id );
					$author_options[$author_id] = "$author->display_name ($author->user_login)";
				}
			}
			
			asort( $author_options );
			
			if ( empty( $author_options ) )
				unset( $views['is_author'] );
			
			
			$form =& new ITForm( $view );
			
?>
	<?php $form->start_form(); ?>
		<table class="valign-top">
			<tr><td><?php _e( 'View', 'it-l10n-Builder' ); ?></td>
				<td>
					<?php $form->add_drop_down( 'view', $views ); ?>
					<div id="view-description"></div>
				</td>
			</tr>
			<tr id="tag-options" style="display:none;"><td><?php _e( 'Tag', 'it-l10n-Builder' ); ?></td>
				<td>
					<?php $form->add_drop_down( 'tag_id', $tag_options ); ?>
				</td>
			</tr>
			<tr id="category-options" style="display:none;"><td><?php _e( 'Category', 'it-l10n-Builder' ); ?></td>
				<td>
					<?php $form->add_drop_down( 'category_id', $category_options ); ?>
				</td>
			</tr>
			<tr id="author-options" style="display:none;"><td><?php _e( 'Author', 'it-l10n-Builder' ); ?></td>
				<td>
					<?php $form->add_drop_down( 'author_id', $author_options ); ?>
				</td>
			</tr>
			<tr><td><?php _e( 'Layout', 'it-l10n-Builder' ); ?></td>
				<td><?php $form->add_drop_down( 'layout', $layouts ); ?>
			</tr>
		</table>
		<?php $form->add_hidden( 'render_clean', '1' ); ?>
		<?php if ( isset( $_REQUEST['original_view'] ) ) $form->add_hidden( 'original_view', $_REQUEST['original_view'] ); ?>
		<br />
		
		<div style="text-align:center;">
			<?php $save_desc = ( empty( $_REQUEST['original_view'] ) ) ? __( 'Add', 'it-l10n-Builder' ) : __( 'Update', 'it-l10n-Builder' ); ?>
			<?php $form->add_submit( 'modify_view', array( 'value' => $save_desc, 'class' => 'button-primary save' ) ); ?>
			<?php $form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary', 'onclick' => 'var win = window.dialogArguments || opener || parent || top; win.remove_new_module(); win.tb_remove(); return false;' ) ); ?>
		</div>
		
		<div id="view-descriptions" style="display:none;">
			<?php foreach ( (array) $available_views as $id => $view ) : ?>
				<div id="view-<?php echo $this->_parse_view_id( $id ); ?>"><?php echo $view['description']; ?></div>
			<?php endforeach; ?>
		</div>
	<?php $form->end_form(); ?>
	<script type="text/javascript">
		/* <![CDATA[ */
		auto_resize_thickbox_height();
		
		init_modify_view_screen();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _modify_views() {
			$available_views = apply_filters( 'builder_get_available_views', array() );
			
			$views = array();
			foreach ( (array) $this->_options['views'] as $view_id => $layout ) {
				$view = $this->_get_view_data( $view_id );
				
				if ( empty( $view ) )
					continue;
				
				$views[$view_id] = $view;
			}
			
			it_classes_load( 'it-array-sort.php' );
			$sorter = new ITArraySort( $views, 'name' );
			$views = $sorter->get_sorted_array();
			
			
			$form =& new ITForm();
			
			$add_link = "$this->_self_link&render_clean=1&modify_view_screen=1&TB_iframe=true&height=100&width=300&modal=true";
			
?>
	<div class="wrap">
		<?php $form->start_form(); ?>
			<p><a href="<?php echo $this->_self_link; ?>"><?php _e( '&laquo; Back to Layouts', 'it-l10n-Builder' ); ?></a></p>
			
			<h2><?php _e( 'Apply Layouts to Specific Views', 'it-l10n-Builder' ); ?></h2>
			
			<div id="no-views-container">
				<p><?php _e( 'No custom views have been added yet. Please click the button below if you would like to apply a specific layout to a site view.', 'it-l10n-Builder' ); ?></p>
				<a href="<?php echo $add_link; ?>" class="button-secondary link-secondary"><?php _e( 'Add View', 'it-l10n-Builder' ); ?></a>
			</div>
			<div id="views-container">
				<div class="tablenav">
					<div class="alignleft actions">
						<a href="<?php echo $add_link; ?>" class="button-secondary link-secondary"><?php _e( 'Add View', 'it-l10n-Builder' ); ?></a>
					</div>
					
					<br class="clear" />
				</div>
				
				<br class="clear" />
				
				<table id="views-table" class="widefat fixed">
					<thead>
						<tr class="thead">
							<th><?php _e( 'Site View', 'it-l10n-Builder' ); ?></th>
							<th><?php _e( 'Site View Description', 'it-l10n-Builder' ); ?></th>
							<th title=""><?php _e( 'Layout', 'it-l10n-Builder' ); ?></th>
						</tr>
					</thead>
					<tfoot>
						<tr class="thead">
							<th><?php _e( 'Site View', 'it-l10n-Builder' ); ?></th>
							<th><?php _e( 'Site View Description', 'it-l10n-Builder' ); ?></th>
							<th title=""><?php _e( 'Layout', 'it-l10n-Builder' ); ?></th>
						</tr>
					</tfoot>
					<tbody>
						<?php $class = ' alternate'; ?>
						<?php foreach ( (array) $views as $view_id => $view ) : ?>
							<tr class="view-entry" id="view-<?php echo $this->_parse_view_id( $view_id ); ?>"<?php echo $class; ?>>
								<td>
									<strong><a class="view-name" href="<?php echo $this->_self_link; ?>&render_clean=1&modify_view_screen=1&original_view=<?php echo $view_id; ?>&TB_iframe=true&height=100&width=350&modal=true" title="<?php _e( 'Modify View', 'it-l10n-Builder' ); ?>"><?php echo $view['name']; ?></a></strong><br />
									<div class="row-actions">
										<span class="edit"><a href="<?php echo $this->_self_link; ?>&render_clean=1&modify_view_screen=1&original_view=<?php echo $view_id; ?>&TB_iframe=true&height=100&width=350&modal=true" title="Modify View"><?php _e( 'Edit', 'it-l10n-Builder' ); ?></a> | </span>
										<span class="delete"><a href="<?php echo $this->_self_link; ?>&render_clean=1&delete_view_screen=<?php echo $view_id; ?>&TB_iframe=true&height=100&width=300" title="Remove View Customization"><?php _e( 'Remove', 'it-l10n-Builder' ); ?></a></span>
									</div>
								</td>
								<td><?php echo $view['description']; ?></td>
								<td class="view-layout">
									<a href="<?php echo $this->_self_link; ?>&layout=<?php echo $view['layout']; ?>" title="<?php _e( 'Modify Layout', 'it-l10n-Builder' ); ?>">
										<?php echo $this->_options['layouts'][$view['layout']]['description']; ?>
									</a>
								</td>
							</tr>
							<?php $class = ( $class == '' ) ? ' alternate' : ''; ?>
						<?php endforeach; ?>
					</tbody>
				</table>
				
				<br class="clear" />
				
				<div class="tablenav">
					<div class="alignleft actions">
						<a href="<?php echo $add_link; ?>" class="button-secondary link-secondary"><?php _e( 'Add View', 'it-l10n-Builder' ); ?></a>
					</div>
					
					<br class="clear" />
				</div>
			</div>
			
			<div id="new-view-container" style="display:none;">
				<table>
					<tr class="view-entry" id="view-%parsed_view_id%" style="display:none;">
						<td>
							<strong><a class="view-name" href="<?php echo $this->_self_link; ?>&render_clean=1&modify_view_screen=1&original_view=%view_id%&TB_iframe=true&height=100&width=350&modal=true" title="<?php _e( 'Modify View', 'it-l10n-Builder' ); ?>">%view_name%</a></strong><br />
							<div class="row-actions">
								<span class="edit"><a href="<?php echo $this->_self_link; ?>&render_clean=1&modify_view_screen=1&original_view=%view_id%&TB_iframe=true&height=100&width=350&modal=true" title="Modify View"><?php _e( 'Edit', 'it-l10n-Builder' ); ?></a> | </span>
								<span class="delete"><a href="<?php echo $this->_self_link; ?>&render_clean=1&delete_view_screen=%view_id%&TB_iframe=true&height=100&width=300" title="Remove View Customization"><?php _e( 'Remove', 'it-l10n-Builder' ); ?></a></span>
							</div>
						</td>
						<td>%view_description%</td>
						<td class="" title="">
							<a href="<?php echo $this->_self_link; ?>&layout=%layout%" title="<?php _e( 'Modify Layout', 'it-l10n-Builder' ); ?>">
								%layout_description%
							</a>
						</td>
					</tr>
				</table>
			</div>
			
			<div id="js-vars" style="display:none;">
				<?php foreach ( (array) $available_views as $view => $data ) : ?>
					<div id="view-name-<?php echo $this->_parse_view_id( $view ); ?>"><?php echo $data['name']; ?></div>
					<div id="view-description-<?php echo $this->_parse_view_id( $view ); ?>"><?php echo $data['description']; ?></div>
				<?php endforeach; ?>
				
				<?php foreach ( (array) $this->_options['layouts'] as $layout => $data ) : ?>
					<div id="layout-description-<?php echo $layout; ?>"><?php echo $data['description']; ?></div>
					<div id="layout-link-href-<?php echo $layout; ?>"><?php echo $this->_self_link; ?>&layout=<?php echo $layout; ?></div>
				<?php endforeach; ?>
			</div>
		<?php $form->end_form(); ?>
	</div>
	
	<script type="text/javascript">
		/* <![CDATA[ */
		jQuery(document).ready( function() { tb_init("a[href*='TB_iframe']"); } );
		init_modify_views();
		/* ]]> */
	</script>
<?php
		}
		
		function _save_layout() {
			$layout = array();
			
			if ( isset( $_POST['layout-guid'] ) )
				$layout['guid'] = $_POST['layout-guid'];
			
			$layout['description'] = $_POST['description'];
			$layout['width'] = $_POST['width'];
			$layout['custom_width'] = $_POST['custom_width'];
			$layout['extension'] = $_POST['extension'];
			$layout['disable_style'] = $_POST['disable_style'];
			$layout['hide_widgets'] = $_POST['hide_widgets'];
			$layout['modules'] = array();
			
			ksort( $_POST );
			foreach ( (array) $_POST as $var => $val ) {
				if ( preg_match( '/^module-(\d+)-(.+)$/', $var, $matches ) )
					$layout['modules'][$_POST["position-{$matches[1]}"]]['data'][$matches[2]] = $val;
				else if ( preg_match( '/^module-(\d+)$/', $var, $matches ) ) {
					$layout['modules'][$_POST["position-{$matches[1]}"]]['module'] = $val;
					
					if ( ! isset( $layout['modules'][$_POST["position-{$matches[1]}"]]['data'] ) )
						$layout['modules'][$_POST["position-{$matches[1]}"]]['data'] = array();
				}
				else if ( preg_match( '/^module-guid-(\d+)$/', $var, $matches ) )
					$layout['modules'][$_POST["position-{$matches[1]}"]]['guid'] = $val;
			}
			
			foreach ( (array) $layout['modules'] as $id => $module ) {
				if ( ! isset( $module['guid'] ) || empty( $module['guid'] ) )
					$layout['modules'][$id]['guid'] = uniqid( '' );
			}
			
			
			ksort( $layout['modules'] );
			
			
			$error = false;
			
			if ( empty( $layout['description'] ) ) {
				ITUtility::show_error_message( __( 'You must supply a Layout Name.', 'it-l10n-Builder' ) );
				$error = true;
			}
			if ( 'custom' === $_POST['width'] ) {
				if ( preg_match( '/[^0-9]/', $_POST['custom_width'] ) ) {
					ITUtility::show_error_message( __( 'The Layout Width must be an integer number.', 'it-l10n-Builder' ) );
					$error = true;
				}
				else if ( $_POST['custom_width'] < 50 ) {
					ITUtility::show_error_message( __( 'The Layout Width must be at least 50 pixels. Please increase the Layout Width.', 'it-l10n-Builder' ) );
					$error = true;
				}
			}
			
			if ( true === $error ) {
				$this->_cached_layout = $layout;
				$this->_modify_layout();
				
				return;
			}
			
			
			if ( ! empty( $_REQUEST['add_layout'] ) ) {
				foreach ( (array) $this->_options['layouts'] as $ex_layout ) {
					if ( strtolower( $layout['description'] ) === strtolower( $ex_layout['description'] ) ) {
						ITUtility::show_error_message( __( 'A layout with that Layout Name already exists. Please choose a unique name.', 'it-l10n-Builder' ) );
						
						$this->_cached_layout = $layout;
						$this->_modify_layout();
						
						return;
					}
				}
				
				$id = uniqid( '' );
				$layout['guid'] = $id;
				
				$_REQUEST['layout'] = $id;
				unset( $_REQUEST['add_layout'] );
				
				$this->_options['layouts'][$id] = $layout;
				
				ITUtility::show_status_message( sprintf( __( '%s created', 'it-l10n-Builder' ), $layout['description'] ) );
			}
			else if ( ! empty( $_REQUEST['layout'] ) ) {
				$this->_options['layouts'][$_REQUEST['layout']] = $layout;
				
				ITUtility::show_status_message( sprintf( __( '%s updated', 'it-l10n-Builder' ), $layout['description'] ) );
			}
			
			do_action( 'builder_editor_save_custom_settings', $layout );
			
			
			$this->_save();
			
			if ( empty( $_REQUEST['save_and_continue'] ) )
				$this->_list_layouts();
			else
				$this->_modify_layout();
		}
		
		function _modify_layout() {
			$layout_widths = array(
				'600'		=> __( 'Narrow (600 pixels)', 'it-l10n-Builder' ),
				'780'		=> __( 'Medium (780 pixels)', 'it-l10n-Builder' ),
				'960'		=> __( 'Wide (960 pixels)', 'it-l10n-Builder' ),
				'custom'	=> __( 'Custom...', 'it-l10n-Builder' ),
			);
			
			
			$extension_data = apply_filters( 'builder_get_extensions', array() );
			$extensions = array( '' => __( '-- No Extension --', 'it-l10n-Builder' ) );
			
			foreach ( (array) $extension_data as $extension )
				$extensions[$extension['directory']] = $extension['name'];
			
			
			$layouts = array();
			foreach ( (array) $this->_options['layouts'] as $layout_id => $layout_data ) {
				if ( isset( $_REQUEST['layout'] ) && ( $layout_id == $_REQUEST['layout'] ) )
					continue;
				$layouts[$layout_id] = $layout_data['description'];
			}
			sort( $layouts );
			
			
			$layout = array( 'guid' => '' );
			
			if ( isset( $this->_cached_layout ) && is_array( $this->_cached_layout ) )
				$layout = $this->_cached_layout;
			else if ( isset( $_REQUEST['layout'] ) && isset( $this->_options['layouts'][$_REQUEST['layout']] ) )
				$layout = $this->_options['layouts'][$_REQUEST['layout']];
			
			$form =& new ITForm( $layout );
			
?>
	<div class="wrap">
		<p><a href="<?php echo $this->_self_link; ?>"><?php _e( '&laquo; Back to Layouts', 'it-l10n-Builder' ); ?></a></p>
		
		<h2><?php _e( 'Layout Editor', $this->_var ); ?></h2>
		
		<?php $form->start_form(); ?>
			<h3 style="margin:0; padding:0;"><?php _e( 'Settings', 'it-l10n-Builder' ); ?></h3>
			<table class="form-table">
				<tr><th scope="row"><label for="description"><?php _e( 'Layout Name', 'it-l10n-Builder' ); ?></label></th>
					<td>
						<?php $form->add_text_box( 'description', array( 'size' => '15', 'maxlength' => '15' ) ); ?><br />
						<?php _e( 'This name helps identify this layout\'s widget areas. Choose a descriptive, short name.', 'it-l10n-Builder' ); ?>
					</td>
				</tr>
				<tr><th scope="row"><label for="width"><?php _e( 'Layout Width', 'it-l10n-Builder' ); ?></label></th>
					<td>
						<?php $form->add_drop_down( 'width', $layout_widths ); ?>
						<div id="layout-width-custom" style="display:none;">
							<label for="custom_width"><?php _e( 'Custom Width', 'it-l10n-Builder' ); ?></label>
							<?php $form->add_text_box( 'custom_width', array( 'size' => '4', 'maxlength' => '5' ) ); ?>
						</div>
					</td>
				</tr>
				<tr><th scope="row"><label for="extension"><?php _e( 'Extension', 'it-l10n-Builder' ); ?></label></th>
					<td>
						<?php $form->add_drop_down( 'extension', $extensions ); ?><br />
						<?php _e( 'Selecting an Extension allows this Layout to modify the theme\'s style.css styling.', 'it-l10n-Builder' ); ?><br />
						<?php _e( 'The files can be found in the theme\'s extensions directory.', 'it-l10n-Builder' ); ?><br />
						<br />
						<?php _e( 'Should this Extension disable the theme\'s style.css file?', 'it-l10n-Builder' ); ?><br />
						<?php _e( 'Selecting Yes with a Extension selected will cause style.css to not be used on this Layout.', 'it-l10n-Builder' ); ?><br />
						<?php $form->add_drop_down( 'disable_style', array( 'no' => __( 'No', 'it-l10n-Builder' ), 'yes' => __( 'Yes', 'it-l10n-Builder' ) ) ); ?>
					</td>
				</tr>
				<tr><th scope="row"><label for="hide_widgets"><?php _e( 'Hide Widget Areas', 'it-l10n-Builder' ); ?></label></th>
					<td>
						<?php $form->add_drop_down( 'hide_widgets', array( 'no' => __( 'No', 'it-l10n-Builder' ), 'yes' => __( 'Yes', 'it-l10n-Builder' ) ) ); ?><br />
						<?php printf( __( 'Use this option to hide this Layout\'s widget areas from the <a href="%s">Appearance &gt; Widgets editor</a>.', 'it-l10n-Builder' ), admin_url( 'widgets.php' ) ); ?><br />
						<?php _e( 'This makes it easier to work with other Layout\'s widget areas.', 'it-l10n-Builder' ); ?>
					</td>
				</tr>
				<?php do_action( 'builder_editor_add_custom_settings', $layout ); ?>
			</table>
			
			<h3>Design</h3>
			<table class="form-table layout-modules">
				<tr class="add-module-help"><td colspan="2"><?php _e( 'In order to start building your layout, please click the Add Module link below.', 'it-l10n-Builder' ); ?></td></tr>
				<?php
					$position = 1;
					$max_id = 0;
					
					if ( isset( $layout['modules'] ) ) {
						foreach ( (array) $layout['modules'] as $id => $module ) {
							if ( false !== $this->_add_module_fields( $module, $id, $position ) )
								$position++;
							
							if ( $id > $max_id )
							$max_id = $id;
						}
					}
				?>
			</table>
			
			<p class="submit">
				<?php $form->add_submit( 'save', array( 'value' => __( 'Save Layout', 'it-l10n-Builder' ), 'class' => 'button-primary' ) ); ?>
				<?php $form->add_submit( 'save_and_continue', array( 'value' => __( 'Save Layout and Continue Editing', 'it-l10n-Builder' ), 'class' => 'button-secondary' ) ); ?>
				<?php $form->add_submit( 'cancel', array( 'value' => __( 'Cancel', 'it-l10n-Builder' ), 'class' => 'button-secondary cancel' ) ); ?>
			</p>
			
			<input type="hidden" name="next-position" value="<?php echo $position; ?>" />
			<input type="hidden" name="current-position" value="0" />
			<input type="hidden" name="next-id" value="<?php echo ( $max_id + 1 ); ?>" />
			<?php $form->add_hidden( 'self-link', $this->_self_link ); ?>
			<?php if ( isset( $_REQUEST['layout'] ) ) $form->add_hidden( 'layout', $_REQUEST['layout'] ); ?>
			<?php if ( isset( $_REQUEST['layout'] ) ) $form->add_hidden( 'layout-guid', $layout['guid'] ); ?>
			<?php if ( isset( $_REQUEST['add_layout'] ) ) $form->add_hidden( 'add_layout', $_REQUEST['add_layout'] ); ?>
			<?php $form->add_hidden( 'base_url', "{$this->_plugin_url}" ); ?>
		<?php $form->end_form(); ?>
	</div>
<?php
			
			foreach ( (array) $this->_modules as $var => $module ) {
				echo "<table id=\"module-editor-$var\" style=\"display:none;\">";
				$this->_add_module_fields( array( 'module' => $var, 'data' => array() ), '%id%', '%position%' );
				echo '</table>';
				
				$form->add_hidden( "module-name-$var", $module->_name );
				$form->add_hidden( "module-editable-$var", ( method_exists( $module, 'edit' ) ) ? '1' : '0' );
				$form->add_hidden( "module-max-$var", $module->_max );
			}
			
			$module_image_paths = array();
			if ( is_dir( "$this->_plugin_path/modules" ) && ( $readdir = opendir( "$this->_plugin_path/modules" ) ) ) {
				while ( ( $module = readdir( $readdir ) ) !== false ) {
					if ( preg_match( '/^\.{1,2}$/', $module ) )
						continue;
					if ( is_dir( "$this->_plugin_path/modules/$module/images" ) )
						$module_image_paths[] = "modules/$module/images";
				}
			}
			
			echo "<div class=\"preload-images\">\n";
			
			foreach ( (array) $module_image_paths as $path ) {
				if ( $readdir = opendir( "$this->_plugin_path/$path" ) ) {
					while ( ( $image = readdir( $readdir ) ) !== false ) {
						if ( is_file( "$this->_plugin_path/$path/$image" ) && preg_match ( '/\.(png|jpg|jpeg|gif)$/i', $image ) )
							echo "<img src=\"$this->_plugin_url/$path/$image\" alt=\"preload image\" />\n";
					}
				}
			}
			
			echo "</div>\n";
			
?>
	<script type="text/javascript">
		/* <![CDATA[ */
		init_layout_editor();
		/* ]]> */
	</script>
<?php
			
		}
		
		function _add_module_fields( $module, $id, $position ) {
			$defaults = $this->_modules[$module['module']]->get_defaults();
			$data = array_merge( $defaults, $module['data'] );
			$form =& new ITForm( $data, array( 'prefix' => "module-$id" ) );
			
			$layout_option = $this->_modules[$module['module']]->get_layout_option();
			
?>
	<tr class="module-row">
		<td class="preview-container"><div id="module-<?php echo $id; ?>-preview" class="module-<?php echo $this->_modules[$module['module']]->_var; ?>"></div></td>
		<td>
			<h4><?php echo $this->_modules[$module['module']]->_name; ?></h4>
			
			<div class="row-actions module-links"></div>
			
			<?php
				foreach ( (array) $data as $var => $val ) {
					$value_array = array( 'value' => $val );
					if ( $var === $layout_option )
						$value_array['class'] = 'layout-option';
					
					$form->add_hidden( $var, $value_array );
					echo "\n";
				}
				
				unset( $form->_args['prefix'] );
			?>
			
			<?php $form->add_hidden( "position-$id", array( 'value' => $position, 'class' => 'module-position' ) ); ?>
			<?php $form->add_hidden( "module-$id", array( 'value' => $module['module'] , 'class' => 'module-var' ) ); ?>
			<?php if ( isset( $module['guid'] ) ) $form->add_hidden( "module-guid-$id", $module['guid'] ); ?>
		</td>
	</tr>
<?php
			
			return true;
		}
		
		
		// Plugin Functions ///////////////////////////
		
		function _addPreviewGroup( $var ) {
			echo '<div id="' . $var . '_layout_editor_preview" class="layout_editor_preview">';
			
			if ( file_exists( $this->_plugin_path . '/images/' . $var . '_' . $this->_options['layouts'][$_REQUEST['layout']]['module'][$var] . '.gif' ) )
				echo '<img src="' . $this->_plugin_url .  '/images/' . $var . '_' . $this->_options['layouts'][$_REQUEST['layout']]['modules'][$var] . '.gif" />';
			
			echo "</div>\n";
		}
		
		function _generate_new_layout_id( $description ) {
			return uniqid();
		}
		
		function _add_layout_views() {
			global $wpdb;
			
			
			foreach ( (array) $this->_options['layouts'] as $id => $layout ) {
				$this->_options['layouts'][$id]['num_views'] = 0;
				$this->_options['layouts'][$id]['num_page_views'] = 0;
				$this->_options['layouts'][$id]['num_post_views'] = 0;
				$this->_options['layouts'][$id]['total_num_views'] = 0;
			}
			
			$fix = false;
			
			foreach ( (array) $this->_options['views'] as $id => $layout ) {
				if ( ! isset( $this->_options['layouts'][$layout] ) || ! is_array( $this->_options['layouts'][$layout] ) ) {
					$this->_options['views'][$id] = $this->_options['default'];
					$layout = $this->_options['default'];
					
					$fix = true;
				}
				
				$this->_options['layouts'][$layout]['num_views']++;
				$this->_options['layouts'][$layout]['total_num_views']++;
			}
			
			if ( true === $fix )
				$this->_save();
			
			
			$post_layouts = $wpdb->get_results( "SELECT {$wpdb->posts}.post_type, {$wpdb->postmeta}.meta_value AS layout, COUNT(*) AS count FROM {$wpdb->postmeta} LEFT JOIN {$wpdb->posts} ON {$wpdb->postmeta}.post_id={$wpdb->posts}.ID WHERE {$wpdb->postmeta}.meta_key='_custom_layout' GROUP BY meta_value, post_type" );
			
			foreach ( (array ) $post_layouts as $post_layout ) {
				if ( ! isset( $this->_options['layouts'][$post_layout->layout]["num_{$post_layout->post_type}_views"] ) )
					continue;
				
				if ( ! isset( $this->_options['layouts'][$post_layout->layout] ) || ! is_array( $this->_options['layouts'][$post_layout->layout] ) ) {
					$wpdb->query( $wpdb->prepare( "UPDATE {$wpdb->postmeta} SET meta_value=%s WHERE meta_key='_custom_layout' AND meta_value=%s", $this->_options['default'], $post_layout->layout ) );
					continue;
				}
				
				$this->_options['layouts'][$post_layout->layout]["num_{$post_layout->post_type}_views"] += $post_layout->count;
				$this->_options['layouts'][$post_layout->layout]['total_num_views'] += $post_layout->count;
			}
		}
		
		function _orderedSort( $a, $b ) {
			return strcasecmp( $this->_options['layouts'][$a]['description'], $this->_options['layouts'][$b]['description'] );
		}
		
		function _parse_view_id( $view ) {
			$view = str_replace( '&', '-', $view );
			$view = str_replace( '|', '_', $view );
			return $view;
		}
		
		function _get_view_data( $view_id ) {
			$available_views = apply_filters( 'builder_get_available_views', array() );
			
			list( $temp_view_id ) = explode( '__', $view_id );
			if ( ! isset( $available_views[$temp_view_id] ) )
				return array();
			
			$layout = $this->_options['views'][$view_id];
			
			if ( strpos( $view_id, '__' ) ) {
				list( $view, $term_id ) = explode( '__', $view_id );
				
				if ( preg_match( '|^(builder_)?is_(.+)$|', $view, $matches ) ) {
					$type = $matches[2];
					$type_description = str_replace( '_', ' ', $type );
				}
				
				if ( 'tag' === $type ) {
					$type = 'post_tag';
					$type_description = 'tag';
				}
				
				if ( 'author' === $type ) {
					$author = get_userdata( $term_id );
					$term_name = "$author->display_name ($author->user_login)";
				}
				else
					$term_name = $this->_get_full_term_name( $term_id, $type );
				
				$name = "{$available_views[$view]['name']} - $term_name";
				
				$category_id = $tag_id = $author_id = null;
				if ( 'is_category' === $view )
					$category_id = $term_id;
				else if ( 'is_tag' === $view )
					$tag_id = $term_id;
				else if ( 'is_author' === $view )
					$author_id = $term_id;
				
				
				$view = array(
					'view_id'				=> $view_id,
					'view'					=> $view,
					'term_id'				=> $term_id,
					'category_id'			=> $category_id,
					'tag_id'				=> $tag_id,
					'author_id'				=> $author_id,
					'name'					=> $name,
					'description'			=> "View for $type_description $term_name",
					'layout'				=> $layout,
					'layout_description'	=> $this->_options['layouts'][$layout]['description'],
				);
			}
			else {
				$view = array(
					'view_id'				=> $view_id,
					'view'					=> $view_id,
					'term_id'				=> null,
					'category_id'			=> null,
					'tag_id'				=> null,
					'author_id'				=> null,
					'name'					=> $available_views[$view_id]['name'],
					'description'			=> $available_views[$view_id]['description'],
					'layout'				=> $layout,
					'layout_description'	=> $this->_options['layouts'][$layout]['description'],
				);
			}
			
			return $view;
		}
		
		function _get_full_term_name( $term_id, $type ) {
			$term = get_term( $term_id, $type );
			$full_name = $term->name;
			
			while ( $term->parent > 0 ) {
				$term = get_term( $term->parent, $term->taxonomy );
				$full_name = "$term->name \ $full_name";
			}
			
			return $full_name;
		}
	}
	
	new BuilderLayoutEditor();
}

?>
