<?php

/*
Written by Chris Jean for iThemes.com
Version 2.1.4

Version History
	1.0.0 - 2009-12-11
		Initial release version
	1.0.1 - 2009-12-15
		Updated add_extension to only output
			if extension is set
	1.0.2 - 2009-12-16
		Fixed bug where core Builder style.css was recognized
			as a Layout Style
	1.0.3 - 2009-12018
		Fixed warnings generated by add_extension
	2.0.0 - 2010-01-07
		Changed from layout-styles.php to extensions.php
		Changed all instances of layout_styles to extensions
	2.1.0 - 2010-01-14
		Added ability for child themes to provide Extensions
	2.1.1 - 2010-01-15
		Fixed functions.php call for child theme Extensions
	2.1.2 - 2010-01-18
		Changed builder_layout_engine_add_extension_stylesheet to builder_extension_add_stylesheet
		Changed builder_layout_engine_run_extension_functions to builder_extension_run_functions
		Changed builder_layout_engine_get_extensions to builder_get_extensions
		Changed builder_layout_engine_get_extension_data to builder_get_extension_data
		Changed builder_layout_engine_get_current_layout to builder_get_current_layout
		Changed builder_layout_engine_get_extension_directories to builder_get_extension_directories
	2.1.3 - 2010-01-18
		Added check to not try to render stylesheet for missing extension
	2.1.4 - 2010-02-22
		Added preg_quote around WP_PLUGIN_DIR and WP_CONTENT_DIR expressions
*/


if ( ! class_exists( 'BuilderExtensions' ) ) {
	class BuilderExtensions {
		var $_extension_directory = 'extensions';
		
		function BuilderExtensions() {
			add_action( 'builder_extension_add_stylesheet', array( &$this, 'add_stylesheet' ) );
			add_action( 'builder_extension_run_functions', array( &$this, 'run_functions' ) );
			
			add_filter( 'builder_get_extensions', array( &$this, 'get_extensions' ) );
			add_filter( 'builder_get_extension_data', array( &$this, 'get_data' ) );
		}
		
		function add_stylesheet( $extension = null ) {
			if ( empty( $extension ) ) {
				$layout = apply_filters( 'builder_get_current_layout', array() );
				
				if ( isset( $layout['extension'] ) )
					$extension = $layout['extension'];
			}
			
			if ( empty( $extension ) )
				return;
			
			$extension_data = $this->get_data( array( 'directory' => $extension ) );
			
			if ( empty( $extension_data['stylesheet_url'] ) )
				return;
			
			echo "<link rel='stylesheet' href='{$extension_data['stylesheet_url']}' type='text/css' media='screen' />\n";
		}
		
		function run_functions( $extension ) {
			$extension_data = $this->get_data( array( 'directory' => $extension ) );
			
			if ( ! empty( $extension_data['functions_path'] ) )
				ITUtility::include_file_once( $extension_data['functions_path'] );
		}
		
		function get_extensions( $extensions ) {
			if ( get_template_directory() !== get_stylesheet_directory() ) {
				$files = glob( get_stylesheet_directory() . '/' . $this->_extension_directory . '/*/style.css' );
				
				if ( ! empty( $files ) )
					$directories['child_theme'] = get_stylesheet_directory() . '/' . $this->_extension_directory;
			}
			if ( ! isset( $directories ) )
				$directories = array( 'default' => get_template_directory() . '/' . $this->_extension_directory );
			
			$directories = apply_filters( 'builder_get_extension_directories', $directories );
			
			foreach ( (array) $directories as $directory ) {
				$new_extensions = $this->_get_directory_extensions( $directory );
				
				$extensions = array_merge( $extensions, $new_extensions );
			}
			
			return $extensions;
		}
		
		function _get_directory_extensions( $extension_directory ) {
			$extension_styles = glob( "$extension_directory/*/style.css" );
			$extensions = array();
			
			foreach ( (array) $extension_styles as $style ) {
				$extension_dir = preg_replace( '|/style\.css$|', '', $style );
				$extension_dir = preg_replace( '|^' . preg_quote( WP_PLUGIN_DIR, '|' ) . '|', '%WP_PLUGIN_DIR%', $extension_dir );
				$extension_dir = preg_replace( '|^' . preg_quote( WP_CONTENT_DIR, '|' ) . '|', '%WP_CONTENT_DIR%', $extension_dir );
				
				$extension = $this->get_data( array( 'directory' => $extension_dir ) );
				
				$extensions[$extension['directory']] = $extension;
			}
			
			return $extensions;
		}
		
		function get_data( $extension_data ) {
			$dir = $url = $extension_data['directory'];
			
			if ( preg_match_all( '/%([^%]+)%/', $extension_data['directory'], $matches, PREG_SET_ORDER ) ) {
				foreach ( (array) $matches as $match ) {
					$dir = preg_replace( "/%{$match[1]}%/", constant( $match[1] ), $dir );
					
					$url_constant = preg_replace( '/_DIR$/', '_URL', $match[1] );
					
					if ( defined( $url_constant ) )
						$url = preg_replace( "/%{$match[1]}%/", constant( $url_constant ), $url );
				}
			}
			
			if ( ! file_exists( "$dir/style.css" ) )
				return $extension_data;
			
			$default_headers = array(
				'Name'			=> 'Name',
				'Description'	=> 'Description',
			);
			
			$data = get_file_data( "$dir/style.css", $default_headers, 'extension' );
			foreach ( (array) $data as $key => $val ) {
				unset( $data[$key] );
				$data[strtolower( $key )] = $val;
			}
			
			if ( empty( $data['name'] ) )
				return $extension_data;
			
			$extension_data = array_merge( $extension_data, $data );
			
			$extension_data['stylesheet_path'] = "$dir/style.css";
			$extension_data['stylesheet_url'] = "$url/style.css";
			
			if ( file_exists( "$dir/functions.php" ) ) {
				$extension_data['functions_path'] = "$dir/functions.php";
				$extension_data['functions_url'] = "$url/functions.php";
			}
			
			return $extension_data;
		}
	}
	
	new BuilderExtensions();
}


?>
