<?php

/*
Written by Chris Jean for iThemes.com
Version 1.0.3

Version History
	1.0.0 - 2010-04-20
		Initial release version
	1.0.1 - 2010-06-02
		Updated builder_custom_caption_shortcode to fix
			caption output
	1.0.2 - 2010-07-16
		Added builder_get_custom_post_types function
	1.0.3 - 2010-08-24
		Added builder_register_module_style function
*/


function it_set_theme_menu_var( $menu_var ) {
	return $GLOBALS['theme_menu_var'];
}
add_filter( 'it_filter_theme_menu_var', 'it_set_theme_menu_var' );

function it_set_theme_index( $theme_index ) {
	return $GLOBALS['theme_index'];
}
add_filter( 'it_storage_filter_theme_index', 'it_set_theme_index' );


// Pass a module of * to register the style for all modules
function builder_register_module_style( $module, $name, $selector ) {
	global $builder_module_styles;
	
	if ( ! isset( $builder_module_styles ) )
		$builder_module_styles = array();
	
	$builder_module_styles[$module][$selector] = $name;
}


// Get current taxonomy term title
function builder_get_tax_term_title() {
	if ( is_tax() ) {
		global $wp_query;
		
		$term = $wp_query->get_queried_object();
		return $term->name;
	}
	
	return '';
}


// Get current post's author link
function builder_get_author_link() {
	global $post;
	
	if ( isset( $post ) )
		return '<a href="' . get_author_posts_url( get_the_author_meta( 'ID' ) ) . '" title="' . esc_attr( get_the_author() ) . '">' . get_the_author() . '</a>';
	return '';
}


// Load the not_found.php file
function builder_template_show_not_found() {
	include( builder_theme_file( 'not_found.php' ) );
}
add_action( 'builder_template_show_not_found', 'builder_template_show_not_found' );


// Customize image shortcode output
// Built from version 2.9.2
function builder_custom_caption_shortcode( $output, $attr, $content ) {
	$defaults = array(
		'id'		=> '',
		'align'		=> 'alignnone',
		'width'		=> '',
		'caption'	=> '',
	);
	extract( shortcode_atts( $defaults, $attr ) );
	
	if ( 1 > (int) $width || empty( $caption ) )
		return $content;
	
	if ( ! empty( $id ) )
		$id = 'id="' . esc_attr( $id ) . '"';
	
	$align = esc_attr( $align );
	
	return "<div $id class='wp-caption $align' style='width:{$width}px;'>" . do_shortcode( $content ) . "<p class='wp-caption-text'>$caption</p></div>";
}
add_filter( 'img_caption_shortcode', 'builder_custom_caption_shortcode', 10, 3 );


// Customize post gallery output
// Built from version 2.9.2
function builder_custom_post_gallery( $output, $attr ) {
	global $post, $wp_locale;
	
	static $instance = 0;
	$instance++;
	
	
	// We're trusting author input, so let's at least make sure it looks like a valid orderby statement
	if ( isset( $attr['orderby'] ) ) {
		$attr['orderby'] = sanitize_sql_orderby( $attr['orderby'] );
		if ( empty( $attr['orderby'] ) )
			unset( $attr['orderby'] );
	}
	
	$defaults = array(
		'order'			=> 'ASC',
		'orderby'		=> 'menu_order ID',
		'id'			=> $post->ID,
		'itemtag'		=> 'dl',
		'icontag'		=> 'dt',
		'captiontag'	=> 'dd',
		'size'			=> 'thumbnail',
		'include'		=> '',
		'exclude'		=> ''
	);
	extract( shortcode_atts( $defaults, $attr ) );
	
	$id = intval( $id );
	if ( 'RAND' === $order )
		$orderby = 'none';
	
	if ( ! empty( $include ) ) {
		$include = preg_replace( '/[^0-9,]+/', '', $include );
		$_attachments = get_posts( array( 'include' => $include, 'post_status' => 'inherit', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $order, 'orderby' => $orderby ) );
		
		$attachments = array();
		foreach ( $_attachments as $key => $val )
			$attachments[$val->ID] = $_attachments[$key];
	}
	else if ( ! empty( $exclude ) ) {
		$exclude = preg_replace( '/[^0-9,]+/', '', $exclude );
		$attachments = get_children( array( 'post_parent' => $id, 'exclude' => $exclude, 'post_status' => 'inherit', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $order, 'orderby' => $orderby ) );
	}
	else {
		$attachments = get_children( array( 'post_parent' => $id, 'post_status' => 'inherit', 'post_type' => 'attachment', 'post_mime_type' => 'image', 'order' => $order, 'orderby' => $orderby ) );
	}
	
	if ( empty( $attachments ) )
		return '';
	
	if ( is_feed() ) {
		$output = "\n";
		foreach ( $attachments as $att_id => $attachment )
			$output .= wp_get_attachment_link( $att_id, $size, true ) . "\n";
		return $output;
	}
	
	$itemtag = tag_escape( $itemtag );
	$captiontag = tag_escape( $captiontag );
	$float = ( 'rtl' === $wp_locale->text_direction ) ? 'right' : 'left';
	
	$selector = "gallery-{$instance}";
	
	$output .= "<div class='gallery clearfix'>\n";
	
	$i = 0;
	foreach ( $attachments as $id => $attachment ) {
		$link = isset($attr['link']) && 'file' == $attr['link'] ? wp_get_attachment_link($id, $size, false, false) : wp_get_attachment_link($id, $size, true, false);
		
		$output .= "<{$itemtag} class='gallery-item'>";
		$output .= "
			<{$icontag} class='gallery-icon'>
				$link
			</{$icontag}>";
		if ( $captiontag && trim($attachment->post_excerpt) ) {
			$output .= "
				<{$captiontag} class='gallery-caption'>
				" . wptexturize($attachment->post_excerpt) . "
				</{$captiontag}>";
		}
		$output .= "</{$itemtag}>";
	}
	
	$output .= "</div>\n";
	
	
	return $output;
}
add_filter( 'post_gallery', 'builder_custom_post_gallery', 10, 2 );


// Do smart comments_popup_link replacement
function builder_comments_popup_link( $before, $after, $format, $zero = '(0)', $one = '(1)', $multi = '(%)' ) {
	global $id;
	
	$number = get_comments_number( $id );
	
	if ( ( ! comments_open() && ! pings_open() ) || post_password_required() )
		return;
	
	ob_start();
	comments_popup_link( $zero, $one, $multi );
	$comments = ob_get_contents();
	ob_end_clean();
	
	echo $before;
	printf( $format, $comments );
	echo $after;
}
add_action( 'builder_comments_popup_link', 'builder_comments_popup_link', 10, 6 );


function builder_theme_file( $file ) {
	ltrim( '/\\', $file );
	
	if ( file_exists( get_stylesheet_directory() . '/' . $file ) )
		return get_stylesheet_directory() . '/' . $file;
	
	return get_template_directory() . '/' . $file;
}


function builder_theme_file_url( $file ) {
	ltrim( '/\\', $file );
	
	if ( file_exists( get_stylesheet_directory() . '/' . $file ) )
		return get_stylesheet_directory_uri() . '/' . $file;
	
	return get_template_directory_uri() . '/' . $file;
}


function builder_get_theme_option( $name ) {
	global $wp_theme_options;
	
	return ( isset( $wp_theme_options[$name] ) ) ? $wp_theme_options[$name] : null;
}


function builder_get_custom_post_types() {
	global $builder_custom_post_types;
	
	if ( isset( $builder_custom_post_types ) )
		return $builder_custom_post_types;
	
	
	global $wp_version;
	
	$builder_custom_post_types = array();
	
	if ( version_compare( $wp_version, '2.9.7', '>' ) ) {
		$custom_post_type_objects = get_post_types( array( 'show_ui' => true, '_builtin' => false ), false );
		
		foreach ( (array) $custom_post_type_objects as $post_type => $settings )
			$builder_custom_post_types[$post_type] = $settings->labels->name;
	}
	else if ( version_compare( $wp_version, '2.8.7', '>' ) ) {
		$custom_post_type_objects = get_post_types();
		$core_post_types = array( 'post', 'page', 'revision', 'attachment' );
		
		foreach ( (array) $custom_post_type_objects as $post_type => $settings ) {
			if ( ! in_array( $post_type, $core_post_types ) )
				$builder_custom_post_types[$post_type] = $post_type;
		}
	}
	
	return $builder_custom_post_types;
}


?>
