<?php

/*
Written by Chris Jean for iThemes.com
Version 2.2.4

Version History
	See history.txt
*/



if ( ! class_exists( 'LayoutModuleNavigation' ) ) {
	class LayoutModuleNavigation extends LayoutModule {
		var $_name = '';
		var $_var = 'navigation';
		var $_description = '';
		var $_editor_width = 400;
		var $_has_sidebars = false;
		
		
		function LayoutModuleNavigation() {
			$this->_name = _x( 'Navigation', 'module', 'it-l10n-Builder' );
			$this->_description = __( 'This module adds a horizontal navigation bar. Category and Page navigations are available.', 'it-l10n-Builder' );
			
			$this->LayoutModule();
		}
		
		function get_layout_option() {
			return 'type';
		}
		
		function get_defaults() {
			return array(
				'type'		=> 'pages',
				'style'		=> '',
			);
		}
		
		function validate() {
			return true;
		}
		
		function _start_table_edit( $form, $results = true ) {
			$types = array(
				'pages'			=> __( 'Builder Settings Pages', 'it-l10n-Builder' ),
				'categories'	=> __( 'Builder Settings Categories', 'it-l10n-Builder' ),
				'wp_legacy'		=> __( 'WordPress Pages', 'it-l10n-Builder' ),
			);
			
			if ( function_exists( 'wp_get_nav_menus' ) ) {
				$menus = wp_get_nav_menus();
				
				foreach ( (array) $menus as $menu )
					$types[$menu->term_id] = sprintf( __( 'Custom Menu - %s', 'it-l10n-Builder' ), $menu->name );
			}
			else {
				$types['wp_legacy'] = __( 'WordPress Pages', 'it-l10n-Builder' );
			}
			
?>
	<tr><td><label for="type"><?php _e( 'Navigation Type', 'it-l10n-Builder' ); ?></label></td>
		<td><?php $form->add_drop_down( 'type', $types ); ?></td>
	</tr>
<?php
			
		}
		
		function _modify_module_inner_wrapper_fields( $fields ) {
			$fields['attributes']['class'][] = "{$fields['class_prefix']}-navigation-{$this->_data['type']}";
			
			return $fields;
		}
		
		function _render( $fields ) {
			global $wp_theme_options;
			
			$data = $fields['data'];
			
			
			if ( is_numeric( $data['type'] ) ) {
				if ( function_exists( 'is_nav_menu' ) && is_nav_menu( $data['type'] ) ) {
					$menu = wp_get_nav_menu_object( $data['type'] );
					
					wp_nav_menu( array( 'menu' => $data['type'], 'container_class' => "menu-{$menu->slug}-container {$fields['class_prefix']}-navigation-menu-wrapper" ) );
				}
			}
			else {
				echo "<div class='{$fields['class_prefix']}-navigation-menu-wrapper'>\n";
				echo "<ul class='menu'>\n";
				
				if ( 'pages' === $data['type'] ) {
					if ( ! empty( $wp_theme_options['include_pages'] ) ) {
						if ( in_array( 'home', (array) $wp_theme_options['include_pages'] ) ) {
							$classes = 'home';
							
							if ( is_front_page() )
								$classes .= ' current_page_item';
							
							$link = get_option( 'home' );
							
							echo "<li class='$classes'><a href='$link'>" . __( 'Home', 'it-l10n-Builder' ) . "</a></li>\n";
						}
						
						if ( in_array( 'site_name', (array) $wp_theme_options['include_pages'] ) ) {
							$classs = 'site-name';
							
							if ( is_front_page() )
								$classes .= ' current_page_item';
							
							$link = get_option( 'home' );
							$link_text = get_bloginfo( 'name' );
							
							echo "<li class='$classes'><a href='$link'>$link_text</a></li>\n";
						}
						
						$include_page_ids = array();
						
						foreach ( (array) $wp_theme_options['include_pages'] as $include_page_id ) {
							if ( is_numeric( $include_page_id ) )
								$include_page_ids[] = $include_page_id;
						}
						
						$exclude_page_ids = apply_filters( 'wp_list_pages_excludes', array() );
						
						$include_page_ids = array_diff( (array) $include_page_ids, (array) $exclude_page_ids );
						
						$include = implode( ',', (array) $include_page_ids );
						$my_pages = "title_li=&depth=0&include=$include";
						
						wp_list_pages( $my_pages );
					}
				}
				else if ( 'categories' === $data['type'] ) {
					if ( ! empty( $wp_theme_options['include_cats'] ) ) {
						$include = implode( ',', (array) $wp_theme_options['include_cats'] );
						$my_cats = "title_li=&depth=0&include=$include";
						
						wp_list_categories( $my_cats );
					}
				}
				else {
					wp_list_pages( 'title_li=' );
				}
				
				echo "</ul>\n</div>\n";
			}
		}
	}
	
	new LayoutModuleNavigation();
}




?>
