<?php

/*
Copyright 2009 iThemes (email: support@ithemes.com)

Written by Chris Jean
Version 1.1.1

Version History
	1.0.1 - 2008-11-07
		Initial release
	1.0.2 - 2008-11-10
		New code milestone
	1.0.3 - 2008-11-17
		Updated to latest _setVars code
	1.0.4 - 2008-11-25
		Switched references to get_option( 'home' ) to get_option( 'siteurl' )
	1.0.5 - 2009-02-12
		Added _addCategoryDropDown function
		Added _getSortedHierarchicalCategories function
	1.0.6 - 2009-11-23
		Fixed array index warnings
	1.1.0 - 2010-02-16
		Removed old form handling code and replaced with ITForm code
	1.1.1 - 2010-09-02
		Changed edit_themes to switch_themes for better multisite support
*/


if ( ! class_exists( 'iThemesThemeOptionsFramework' ) ) {
	class iThemesThemeOptionsFramework {
		var $_var = 'theme-options';
		var $_name = 'Theme Settings';
		var $_version = '1.1.1';
		var $_page = 'ithemes-theme-options';
		var $_nonce = 'theme-options';
		
		var $_class = '';
		var $_initialized = false;
		var $_options = array();
		var $defaults = array();
		var $force_defaults = array();
		var $_pageRef = '';
		
		var $_userID = 0;
		var $_usedInputs = array();
		var $_selectedVars = array();
		var $_pluginPath = '';
		var $_pluginRelativePath = '';
		var $_pluginURL = '';
		var $_themePath = '';
		var $_themeRelativePath = '';
		var $_themeURL = '';
		
		
		function iThemesThemeOptionsFramework() {
			global $wp_theme_name, $wp_theme_shortname, $wp_theme_page_name;
			
			if ( ! empty( $wp_theme_shortname ) )
				$this->_var = "$wp_theme_shortname-options";
			if ( ! empty( $wp_theme_name ) )
				$this->_name = "$wp_theme_name Theme Settings";
			
			$this->_setVars();
			
			add_action( 'init', array( &$this, '_init' ), 50 );
			add_action( 'ithemes_save', array( &$this, '_savePluginData' ), 10, 2 );
			
			if ( method_exists( $this, 'load' ) )
				$this->load();
		}
		
		
		function _init() {
			do_action( 'ithemes_load_plugins', $this );
			
			$this->_load();
			
			do_action( 'ithemes_init', $this );
			
			if ( method_exists( $this, 'init' ) )
				$this->init();
			
			add_action( 'admin_menu', array( &$this, '_addPages' ), -10 );
		}
		
		function _addPages() {
			global $wp_theme_name, $wp_theme_page_name;
			
			if ( ! empty( $wp_theme_page_name ) )
				$this->_pageRef = add_submenu_page( $wp_theme_page_name, 'Settings', 'Settings', 'switch_themes', $this->_page, array( &$this, '_index' ) );
			else
				$this->_pageRef = add_theme_page( "$wp_theme_name Settings", "$wp_theme_name Settings", 'switch_themes', $this->_page, array( &$this, '_index' ) );
			
			add_action( 'admin_print_scripts-' . $this->_pageRef, array( $this, '_addScripts' ) );
			add_action( 'admin_print_styles-' . $this->_pageRef, array( $this, '_addStyles' ) );
			
			if ( method_exists( $this, 'addPages' ) )
				$this->addPages();
		}
		
		function _addScripts() {
			if ( method_exists( $this, 'addScripts' ) )
				$this->addScripts();
		}
		
		function _addStyles() {
			if ( method_exists( $this, 'addStyles' ) )
				$this->addStyles();
		}
		
		function _setVars() {
			$this->_class = get_class( $this );
			
			$user = wp_get_current_user();
			$this->_userID = $user->ID;
			
			
			$this->_pluginPath = dirname( __FILE__ );
			$this->_pluginRelativePath = ltrim( str_replace( '\\', '/', str_replace( rtrim( ABSPATH, '\\\/' ), '', $this->_pluginPath ) ), '\\\/' );
			$this->_pluginURL = get_option( 'siteurl' ) . '/' . $this->_pluginRelativePath;
			
			$this->_themePath = TEMPLATEPATH;
			$this->_themeRelativePath = ltrim( str_replace( '\\', '/', str_replace( rtrim( ABSPATH, '\\\/' ), '', $this->_themePath ) ), '\\\/' );
			$this->_themeURL = get_template_directory_uri();
			
			global $ithemes_theme_path, $ithemes_theme_relative_path, $ithemes_theme_url;
			$ithemes_theme_path = $this->_themePath;
			$ithemes_theme_relative_path = $this->_themeRelativePath;
			$ithemes_theme_url = $this->_themeURL;
		}
		
		
		// Options Storage ////////////////////////////
		
		function _initializeOptions() {
			$this->_options = array();
			
			$this->_save();
		}
		
		function updateDefaults( $defaults ) {
			$this->defaults = $defaults;
		}
		
		function updateForceDefaults( $force_defaults ) {
			$this->force_defaults = $force_defaults;
		}
		
		function _setDefaults() {
			if ( method_exists( $this, 'setDefaults' ) )
				$this->setDefaults();
			
			do_action( 'ithemes_set_defaults', $this );
			
			$this->defaults = $this->_merge_defaults( $this->defaults, $this->force_defaults );
			
			if ( ! is_array( $this->defaults ) )
				$this->defaults = array();
			
			$this->_options = $this->_merge_defaults( $this->_options, $this->defaults );
			$this->_options = $this->_merge_defaults( $this->_options, $this->force_defaults, true );
			
			$this->_save();
		}
		
		function _savePluginData( $name, $options = false ) {
			if ( false === $options )
				unset( $this->_options[$name] );
			else
				$this->_options[$name] = $options;
			
			$this->_save();
		}
		
		function _save() {
			if ( $this->_options == @get_option( $this->_var ) )
				return true;
			
			$GLOBALS['wp_theme_options'] = $this->_options;
			
			return @update_option( $this->_var, $this->_options );
		}
		
		function _load() {
			$data = @get_option( $this->_var );
			
			if ( is_array( $data ) )
				$this->_options = $data;
			else
				$this->_initializeOptions();
			
			$this->_setDefaults();
			
			if ( method_exists( $this, 'afterLoad' ) )
				$this->afterLoad();
			
			$GLOBALS['wp_theme_options'] = $this->_options;
		}
		
		
		// Pages //////////////////////////////////////
		
		function _index() {
			if ( ! empty( $_REQUEST['_wpnonce'] ) )
				ITForm::check_nonce( $this->_nonce );
			
			if ( isset( $_POST['form-save'] ) )
				$this->_save_form();
			else if ( isset( $_POST['reset'] ) )
				$this->_reset_data();
			
			$this->_render_form();
		}
		
		function _reset_data() {
			delete_option( $this->_var );
			$this->_load();
			
			ITUtility::show_status_message( "Theme Options Reset" );
		}
		
		function _save_form() {
			foreach ( (array) explode( ',', $_POST['used-inputs'] ) as $input_name ) {
				$name = str_replace( '[]', '', $input_name );
				
				if ( ! isset( $_POST[$name] ) ) {
					if ( preg_match( '/\[\]$/', $input_name ) )
						$_POST[$name] = array();
					else
						$_POST[$name] = '';
				}
				
				$var_name = preg_replace( '/^' . $this->_var . '-/', '', $name );
				
				if ( is_array( $_POST[$name] ) )
					$this->_options[$var_name] = $_POST[$name];
				else
					$this->_options[$var_name] = stripslashes( $_POST[$name] );
			}
			
			if ( method_exists( $this, 'beforeSaveForm' ) )
				$this->beforeSaveForm();
			
			$this->_save();
			
			ITUtility::show_status_message( "Theme Settings Saved" );
			
			if ( method_exists( $this, 'afterSaveForm' ) )
				$this->afterSaveForm();
		}
		
		function _render_form() {
			$this->_form =& new ITForm( $this->_options, array( 'prefix' => $this->_var ) );
			$form =& $this->_form;
			
			
			if ( method_exists( $this, 'beforeRenderForm' ) )
				$this->beforeRenderForm();
			
?>
	<div style="display:none;">
		<!--<?php print_r( $this->_options ); ?>-->
	</div>
	
	<div class="wrap">
		<?php screen_icon(); ?>
		<h2><?php echo $this->_name; ?></h2>
		
		<?php $form->start_form( array(), $this->_nonce ); ?>
			<table class="form-table">
				<tbody>
					<?php
						if ( method_exists( $this, 'renderForm' ) )
							$this->renderForm();
					?>
				</tbody>
			</table>
			
			<p class="submit">
				<?php $form->add_submit( 'form-save', array( 'value' => 'Save Options &raquo;', 'class' => 'button-primary save' ) ); ?>
			</p>
			
			<p class="submit">
				<?php $form->add_submit( 'reset', array( 'value' => 'Reset Options &raquo;', 'class' => 'button-secondary', 'onclick' => "javascript:if(!confirm('Are you sure that you wish to reset all theme options to default values?')) return false;" ) ); ?>
			</p>
		<?php $form->end_form(); ?>
		</form>
	</div>
<?php
			
			if ( method_exists( $this, 'afterRenderForm' ) )
				$this->afterRenderForm();
		}
		
		
		// Plugin Functions ///////////////////////////
		
		function _merge_defaults( $values, $defaults, $force = false ) {
			if ( ! $this->_is_associative_array( $defaults ) ) {
				if ( ! isset( $values ) )
					return $defaults;
				
				if ( false === $force )
					return $values;
				
				if ( isset( $values ) || is_array( $values ) )
					return $values;
				return $defaults;
			}
			
			foreach ( (array) $defaults as $key => $val ) {
				if ( ! isset( $values[$key] ) )
					$values[$key] = null;
				
				$values[$key] = $this->_merge_defaults($values[$key], $val, $force );
			}
			
			return $values;
		}
		
		function _is_associative_array( &$array ) {
			if ( ! is_array( $array ) || empty( $array ) )
				return false;
			
			$next = 0;
			
			foreach ( $array as $k => $v )
				if ( $k !== $next++ )
					return true;
			
			return false;
		}
		
		function _get_sorted_hierarchical_categories( &$retval, $parent = 0, $depth = 0 ) {
			$categories = get_categories( "hide_empty=0&orderby=name&child_of=$parent" );
			
			if ( empty( $categories ) )
				return array();
			
			foreach ( (array) $categories as $category ) {
				if ( $category->parent != $parent )
					continue;
				
				$retval[] = array( 'name' => $category->name, 'id' => $category->cat_ID, 'depth' => $depth, 'parent' => $category->parent );
				
				$this->_get_sorted_hierarchical_categories( $retval, $category->term_id, $depth + 1 );
			}
			
			return $retval;
		}
	}
}

?>
