<?php

/*

Written by Chris Jean
Version 1.2.1

Version History
	1.0.1 - 2008-11-07
		Initial Release
	1.0.2 - 2008-11-17
		Added PHP4 object compatibility
	1.0.3 - 2008-11-24
		Fixed background uploader link in order to prevent XSS problems caused
		when the WordPress URL and the Blog URL addresses use different domains.
	1.1.0 - 2009-02-12
		Fixed the Category Menu Builder sorting
		Added Site Name to Page Menu Builder
	1.1.1 - 2009-02-13
		Added favicon uploader
	1.1.2 - 2009-06-23
		Removed favicon uploader
	1.2.0 - 2010-02-16
		Added hooks to allow for settings page customizations by child themes
		
		Added builder_filter_default_settings filter
		Added builder_custom_settings action
	1.2.1 - 2010-02-17
		Removed unnecessary afterRenderForm function
*/


$GLOBALS['wp_theme_name']		= 'Builder';
$GLOBALS['wp_theme_shortname']	= 'builder';
$GLOBALS['wp_theme_page_name']	= 'ithemes-builder-theme';

require_once( 'theme-options-framework.php' );

if ( ! class_exists( 'iThemesThemeOptions' ) && class_exists( 'iThemesThemeOptionsFramework' ) ) {
	class iThemesThemeOptions extends iThemesThemeOptionsFramework {
		function setDefaults() {
			$this->force_defaults['include_pages'] = array( 'home' );
			$this->force_defaults['include_cats'] = array();
			$this->force_defaults['tracking_pos'] = 'footer';
			$this->force_defaults['tag_as_keyword'] = 'yes';
			$this->force_defaults['cat_index'] = 'no';
			$this->force_defaults['identify_widget_areas'] = 'yes';
			
			$this->force_defaults = apply_filters( 'builder_filter_default_settings', $this->force_defaults );
		}
		
		function renderForm() {
			$form =& $this->_form;
			
?>
	<tr><th scope="row">Menu Builder</th>
		<td><div>Please select the pages and categories you would like to <strong>INCLUDE</strong> in the Header Menus.</div>
			<table>
				<tr><th style="border:none; padding:0px;"><strong>Pages:</strong></th>
					<th style="border:none; padding:0px 0px 0px 20px;"><strong>Categories:</strong></th>
				</tr>
				<tr><td style="border-bottom:none; vertical-align:top; padding:0px;"><?php $this->create_menu_builder_checkboxes( 'include_pages', 'pages' ); ?></td>
					<td style="border-bottom:none; vertical-align:top; padding:0px 0px 0px 20px;"><?php $this->create_menu_builder_checkboxes( 'include_cats', 'categories' ); ?></td>
				</tr>
			</table>
		</td>
	</tr>
	
	<?php do_action( 'builder_custom_settings', $form ); ?>
	
	<tr><th scope="row">Identify Widget Areas</th>
		<td><div>This option fills in widget areas with default text to make identifying the different widget areas easier.</div>
			<div>Once you have your layout and widgets set up as desired, we recommend that you change this setting to No.</div>
			<?php $form->add_drop_down( 'identify_widget_areas', array( 'yes' => 'Yes (default)', 'no' => 'No' ) ); ?>
		</td>
	</tr>
	
	<tr><th scope="row">Tracking Code</th>
		<td>If you use a tracking service like <a href="http://google.com/analytics">Google Analytics</a>, paste the tracking code in the box below:<br />
			(leave blank for none)<br />
			<?php $form->add_text_area( 'tracking', array( 'rows' => '3', 'cols' => '50' ) ); ?><br />
			Does your tracking service go in the header or footer of the code?<br />
			<?php $form->add_drop_down( 'tracking_pos', array( 'footer' => 'Footer (default)', 'header' => 'Header' ) ); ?>
		</td>
	</tr>
	
	<tr><th scope="row">Search Engine Optimization</th>
		<td>
			Would You like to use post tags as <a href="http://en.wikipedia.org/wiki/Meta_element#The_keywords_attribute" target="_blank">META keywords</a> on single posts? (recommended)<br />
			<?php $form->add_drop_down( 'tag_as_keyword', array( 'yes' => 'Yes (default)', 'no' => 'No' ) ); ?><br />
			<strong>NOTE:</strong> By default, this theme uses either <a href="http://codex.wordpress.org/Template_Tags/the_excerpt_rss" target="_blank">the excerpt</a> on single posts or pages,<br />
			or the blog <a href="<?php bloginfo('wpurl'); ?>/wp-admin/options-general.php" target="_blank">tagline</a> on all other pages (home, archives, etc.) as the <a href="http://en.wikipedia.org/wiki/Meta_element#The_description_attribute" target="_blank">META description</a>.<br /><br />
			
			Would you like your category archives to be indexed by search engines? (<strong>not</strong> recommended)<br />
			<?php $form->add_drop_down( 'cat_index', array( 'no' => 'No (default)', 'yes' => 'Yes' ) ); ?><br />
			<strong>NOTE:</strong> No date based archives or search results will be indexed by Search Engines.
		</td>
	</tr>
<?php
			
		}
		
		function create_menu_builder_checkboxes( $var, $type ) {
			if ( empty( $this->_options[$var] ) )
				$this->_options[$var] = array();
			
			$options = array();
			
			if ( 'pages' == $type ) {
				$options['home'] = array( 'title' => 'Home', 'depth' => 0 );
				$options['site_name'] = array( 'title' => 'Site Name', 'depth' => 0 );
				$source_options = get_pages();
			}
			elseif ( 'categories' == $type ) {
				$source_options = array();
				$this->_get_sorted_hierarchical_categories( $source_options );
			}
			
			
			foreach ( (array) $source_options as $option ) {
				if ( 'pages' == $type ) {
					$parent = $option->post_parent;
					$title = $option->post_title;
					$id = $option->ID;
				}
				elseif ( 'categories' == $type ) {
					$parent = $option['parent'];
					$title = $option['name'];
					$id = $option['id'];
				}
				
				if ( 0 == $parent )
					$options[$id] = array( 'title' => $title, 'depth' => 0 );
				else
					$options[$id] = array( 'title' => $title, 'depth' => ( $options[$parent]['depth'] + 1 ) );
			}
			
			foreach ( (array) $options as $id => $data ) {
				$attributes = array();
				$attributes['value'] = $id;
				
				if ( in_array( $id, $this->_options[$var] ) )
					$attributes['checked'] = 'checked';
				?>
					<div style="position:relative; left:<?php echo ( $data['depth'] * 15 ); ?>px;"><?php $this->_form->add_multi_check_box( $var, $attributes ); ?> <?php echo $data['title']; ?></div>
				<?php
			}
		}
	}
}


if ( empty( $ithemes_theme_options ) )
	$GLOBALS['ithemes_theme_options'] =& new iThemesThemeOptions();

?>
